@@ -52,8 +52,7 @@ typedef struct {
 	gdouble q[4];				/* 回転の状態 (quaternion) */
 	gdouble tilt;				/* 0 〜 90 地球の中心を向いているのが 0 */
 	gdouble dist;				/* 凝視している地表までの距離(m) */
-	/* world wind では地表からの距離になっているが、ここでは
-	 * 標高ゼロの点からの距離 */
+	/* world wind では地表からの距離になっているが、ここでは標高ゼロの点からの距離 */
 
 	gdouble q_target[4];		/* 回転の状態 (quaternion) */
 	gdouble tilt_target;		/* 0 〜 90 地球の中心を向いているのが 0 */
@@ -71,12 +70,18 @@ typedef struct {
 
 	gdouble alt;				/* 高度(m) */
 
-	gdouble deg_view_lon;
-	gdouble deg_view_lat;		/* tilt なしの状態で真上から見たときに見える範囲(degree)
-								   本当なら緯度によって変わる。
-								   それ以前の問題として、ものすごく簡易計算。 */
+	/* tilt なしの状態で真上から見たときに見える範囲(degree) */
+	gdouble deg_view_lon_l;		/* 地平線近くまで含めた広い版 */
+	gdouble deg_view_lat_l;
+	gdouble deg_view_lon_s;		/* 真上から見たときに画面に収まる程度＋αという狭い版 */
+	gdouble deg_view_lat_s;
+
+	/* 真上から見たときに画面がおさまる程度の円の半径 */
+	gdouble deg_view_r;
 
 	gboolean inertia;
+
+	gdouble fovy;
 } Camera;
 
 static Camera *camera;
@@ -93,7 +98,7 @@ calc_camera (void)
 	gdouble yaw_target;
 	gdouble pitch_target;
 	gdouble roll_target;
-	gdouble r;
+	gdouble r = mmap_get_world_radius ();
 
 	quaternion_to_matrix (camera->q, camera->rt);	/* クォータニオンから回転の変換行列を求める */
 
@@ -111,11 +116,8 @@ calc_camera (void)
 		camera->heading -= 360.0;
 	}
 
-	r = mmap_get_world_radius ();
-
 	if (camera->type == CAMERA_TYPE_NWW) {
-		camera->alt =
-			sqrt (camera->dist * camera->dist + r * r + 2.0 * camera->dist * r * cos (RAD (camera->tilt))) - r;
+		camera->alt = sqrt (camera->dist * camera->dist + r * r + 2.0 * camera->dist * r * cos (RAD (camera->tilt))) - r;
 	} else {
 		camera->alt = camera->dist;
 	}
@@ -133,41 +135,52 @@ calc_camera (void)
 
 	{
 		/* 視野範囲は、最終目的地 (**_target) から計算する。 */
-		gdouble l;
-		gdouble m;
-		gdouble angle;
+		gdouble l;				/* 真下を見下ろした時の視界の距離。ものすごくテキトーな計算 */
+		gdouble m;				/* 緯度が上がるにしたがい経度方向の幅を増やすための係数 */
+		gdouble k;				/* 高度を下げるにしたがい、視野範囲をしぼるための係数 */
+		gdouble angle;			/* 地平線まで全部表示するための角度 */
 
-		angle = 65.0;
-		/*
-		   if (camera->dist_target > 100000.0) {
-		   angle = 65.0;
-		   } else {
-		   angle = 80.0;
-		   }
-		 */
-
-		/* 真下を見下ろした時の視界の距離。ものすごくテキトーな計算 */
-		l = 2.0 * camera->dist_target * tan (RAD (angle / 2.0));
-
-		camera->deg_view_lat = 2.0 * DEG (atan (l / 2.0 / r));
+		l = 2.0 * camera->dist_target * tan (RAD (camera->fovy / 2.0));
+		angle = acos (r / (r + camera->dist_target));
 
 		m = cos (RAD (camera->lat_target));
 		if (m < 0.2) {
 			m = 0.2;
-		} else {
-			;
-		}
-		/*
-		   g_print ("angle:%.2f  m:%.2f\n", angle, m);
-		 */
-		camera->deg_view_lon = camera->deg_view_lat / m;
-		if (camera->deg_view_lon >= 360.0) {
-			/* 360 だと幅ゼロだと見られてしまうの少し小さくする */
-			camera->deg_view_lon = 359.9;
 		}
 
+		{
+			gdouble a = 0.9;	/* 高度 3*r の時の係数 */
+			gdouble b = 0.2;	/* 高度 0 の時の係数 */
+			if (camera->dist_target > 3.0 * r) {
+				k = a;
+			} else {
+				k = (a - b) / (3.0 * r) * camera->dist_target + b;
+			}
+		}
+		camera->deg_view_lat_l = 2.0 * DEG (angle) * k;
+		camera->deg_view_lon_l = camera->deg_view_lat_l / m;
+
+		camera->deg_view_r = 2.0 * DEG (atan (l / 2.0 / r));
+		camera->deg_view_lat_s = camera->deg_view_r * 1.6;
+		camera->deg_view_lon_s = camera->deg_view_lat_s / m;
+
+		if (camera->deg_view_lat_s > camera->deg_view_lat_l) {
+			camera->deg_view_lat_s = camera->deg_view_lat_l;
+		}
+		if (camera->deg_view_lon_s > camera->deg_view_lon_l) {
+			camera->deg_view_lon_s = camera->deg_view_lon_l;
+		}
+
+		/* 360 だと幅ゼロだと見られてしまうの少し小さくする */
+		if (camera->deg_view_lon_l >= 360.0) {
+			camera->deg_view_lon_l = 359.9;
+		}
+		if (camera->deg_view_lon_s >= 360.0) {
+			camera->deg_view_lon_s = 359.9;
+		}
 		/*
-		   g_print ("deg_view:lon:%.2f  lat:%.2f\n", camera->deg_view_lon, camera->deg_view_lat);
+		   g_print ("deg_view:l:lon:%.2f  lat:%.2f\n", camera->deg_view_lon_l, camera->deg_view_lat_l);
+		   g_print ("deg_view:s:lon:%.2f  lat:%.2f\n", camera->deg_view_lon_s, camera->deg_view_lat_s);
 		 */
 	}
 }
@@ -256,12 +269,12 @@ camera_reset (void)
 static gboolean
 motion_sub (gdouble * target, gdouble * current, gdouble limit, gdouble point, gdouble step, gdouble multi)
 {
-	gboolean unfinished = FALSE;	/* 半端でまだ描画が必要な場合に TRUE */
+	gboolean finished = FALSE;	/* 目的の範囲に入ったら TRUE */
 	gdouble diff;
 
 	diff = *target - *current;
 	if (diff == 0.0) {
-		return FALSE;
+		return TRUE;
 	}
 
 	DB_SUB (g_print ("motion_SUB:target:%.2f current:%.2f diff:%.2f\t", camera->tilt_target, camera->tilt, diff));
@@ -277,7 +290,7 @@ motion_sub (gdouble * target, gdouble * current, gdouble limit, gdouble point, g
 			DB_SUB (g_print (":diff:"));
 		}
 
-		unfinished = TRUE;
+		finished = FALSE;
 	} else if (diff < -limit) {
 		if (diff < -point) {
 			*current -= step;
@@ -289,36 +302,33 @@ motion_sub (gdouble * target, gdouble * current, gdouble limit, gdouble point, g
 			DB_SUB (g_print (":diff:"));
 		}
 
-		unfinished = TRUE;
+		finished = FALSE;
 	} else {
-		*current = *target;
+		*target = *current;
 
 		DB_SUB (g_print (":    :"));
-		unfinished = FALSE;
+		finished = TRUE;
 	}
 	DB_SUB (g_print ("\n"));
 
-	return unfinished;
+	return finished;
 }
 
-/*****************************************************/
-
-
 /*****************************************************/
 
 static gboolean
 motion_tilt (void)
 {
-	gboolean unfinished = FALSE;	/* 半端でまだ描画が必要な場合に TRUE */
+	gboolean finished = FALSE;	/* 目的の範囲に入ったら TRUE */
 
 	gdouble limit = 0.1;
 	gdouble point = 15.0;		/* 減速の開始位置 */
 	gdouble step = 1.5;
 	gdouble multi = 0.1;
 
-	unfinished = motion_sub (&(camera->tilt_target), &(camera->tilt), limit, point, step, multi);
+	finished = motion_sub (&(camera->tilt_target), &(camera->tilt), limit, point, step, multi);
 
-	return unfinished;
+	return finished;
 }
 
 static gdouble
@@ -435,12 +445,12 @@ slerp (gdouble p[], const gdouble q[], const gdouble r[], const gdouble t)
 static gboolean
 motion_rotate (void)
 {
-	gboolean unfinished = FALSE;	/* 半端でまだ描画が必要な場合に TRUE */
+	gboolean finished = FALSE;	/* 目的の範囲に入ったら TRUE */
 	gdouble q_new[4];
 
 	if (camera->q[0] == camera->q_target[0] && camera->q[1] == camera->q_target[1] &&
 		camera->q[2] == camera->q_target[2] && camera->q[3] == camera->q_target[3]) {
-		return FALSE;
+		return TRUE;
 	}
 
 	slerp (q_new, camera->q, camera->q_target, 0.09);
@@ -477,19 +487,19 @@ g_print("motion_rotate:(%.2f, %.2f)  (%.2f, %.2f)   d:%.2f d_deg:%.2f d_roll:%.2
 */
 
 		if (d < d_deg && d_roll < 0.1) {
-			/* 現在値が目的地に十分近付いたので、現在値を目的地で上書きする */
-			camera->q[0] = camera->q_target[0];
-			camera->q[1] = camera->q_target[1];
-			camera->q[2] = camera->q_target[2];
-			camera->q[3] = camera->q_target[3];
+			/* 現在値が目的地に十分近付いたので、目的地を現在値で上書きする */
+			camera->q_target[0] = camera->q[0];
+			camera->q_target[1] = camera->q[1];
+			camera->q_target[2] = camera->q[2];
+			camera->q_target[3] = camera->q[3];
 
-			unfinished = FALSE;
+			finished = TRUE;
 		} else {
-			unfinished = TRUE;
+			finished = FALSE;
 		}
 	}
 
-	return unfinished;
+	return finished;
 }
 
 void
@@ -512,7 +522,6 @@ camera_mult_quaternion (gdouble q[])
 		camera->q[2] = q_new[2];
 		camera->q[3] = q_new[3];
 	}
-
 	/*
 	   g_print ("camera_mult_quaternion:%.2f %.2f %.2f %.2f\n",
 	   camera->q_target[0], camera->q_target[1], camera->q_target[2], camera->q_target[3]);
@@ -524,11 +533,9 @@ camera_rotate (gdouble yaw, gdouble pitch, gdouble roll)
 {
 	gdouble q_new[4];
 	gdouble q_rot[4];
-
 	/*
 	   g_print("camera_rotate:yaw:%.2f pitch:%.2f roll:%.2f\n",yaw, pitch, roll);
 	 */
-
 	euler_to_quaternion (yaw, pitch, roll, q_rot);
 
 	qmul (q_rot, camera->q_target, q_new);
@@ -546,7 +553,6 @@ camera_rotate (gdouble yaw, gdouble pitch, gdouble roll)
 		camera->q[2] = q_new[2];
 		camera->q[3] = q_new[3];
 	}
-
 }
 
 /*****************************************************/
@@ -559,14 +565,14 @@ camera_rotate (gdouble yaw, gdouble pitch, gdouble roll)
 static gboolean
 motion_dist (void)
 {
-	gboolean unfinished = FALSE;	/* 半端でまだ描画が必要な場合に TRUE */
+	gboolean finished = FALSE;	/* 目的の範囲に入ったら TRUE */
 	gdouble diff = 0.0;
 
 	DB_DIST (g_print ("motion_dist:new_dist:%.2f  current_dist:%.2f \t", camera->dist_target, camera->dist));
 
 	diff = camera->dist_target - camera->dist;
 	if (diff == 0.0) {
-		return FALSE;
+		return TRUE;
 	}
 
 	if (diff > 0.001 * camera->dist_target) {
@@ -577,9 +583,9 @@ motion_dist (void)
 			camera->dist += diff * 0.3;
 			DB_DIST (g_print (":+=diff:"));
 		}
-		unfinished = TRUE;
+		finished = FALSE;
 
-		DB_DIST (g_print (":unfinished:diff:%.2f\n", diff));
+		DB_DIST (g_print (":finished:diff:%.2f\n", diff));
 	} else if (diff < -0.001 * camera->dist_target) {
 		if (diff * 0.3 < -0.03 * camera->dist) {
 			camera->dist *= 0.97;
@@ -588,18 +594,18 @@ motion_dist (void)
 			camera->dist += diff * 0.3;
 			DB_DIST (g_print (":+=diff:"));
 		}
-		unfinished = TRUE;
+		finished = FALSE;
 
-		DB_DIST (g_print (":unfinished:diff:%.2f\n", diff));
+		DB_DIST (g_print (":finished:diff:%.2f\n", diff));
 	} else {
-		camera->dist = camera->dist_target;
-		unfinished = FALSE;
+		camera->dist_target = camera->dist;
+		finished = TRUE;
 
 		DB_DIST (g_print (":  finished:diff:%.2f\n", diff));
 	}
 
 
-	return unfinished;
+	return finished;
 }
 
 void
@@ -612,6 +618,14 @@ camera_set_dist (gdouble dist)
 	}
 }
 
+/* world が変わった時などの、のんびり上がってられない時にはこちらを使う */
+void
+camera_set_dist_direct (gdouble dist)
+{
+	camera->dist_target = dist;
+	camera->dist = camera->dist_target;
+}
+
 void
 camera_dist_up (void)
 {
@@ -635,31 +649,46 @@ camera_dist_down (void)
 gboolean
 camera_next_step (void)
 {
-	gboolean unfinished_dist = FALSE;	/* 半端でまだ描画が必要な場合に TRUE */
-	gboolean unfinished_tilt = FALSE;
-	gboolean unfinished_rotate = FALSE;
+	gboolean finished_dist = FALSE;	/* 目的の範囲に入ったら TRUE */
+	gboolean finished_tilt = FALSE;
+	gboolean finished_rotate = FALSE;
 
-	unfinished_dist = motion_dist ();
-	unfinished_tilt = motion_tilt ();
-	unfinished_rotate = motion_rotate ();
+	finished_dist = motion_dist ();
+	finished_tilt = motion_tilt ();
+	finished_rotate = motion_rotate ();
 
 	calc_camera ();
 
-	/*
-	   util_print_bool("dist:", unfinished_dist);
-	   util_print_bool("tilt:", unfinished_tilt);
-	   util_print_bool("rotate:", unfinished_rotate);
+	/*  
+	   util_print_bool("dist:", finished_dist);
+	   util_print_bool("tilt:", finished_tilt);
+	   util_print_bool("rotate:", finished_rotate);
 	 */
 
-	return unfinished_dist || unfinished_tilt || unfinished_rotate;
+	/* 全部が目的の範囲に入ったら TRUE を返す */
+	return finished_dist && finished_tilt && finished_rotate;
 }
 
 /*****************************************************/
 
 void
-camera_matrix_prepare (void)
+camera_matrix_prepare (gint width, gint height)
 {
+	gdouble l = 1000.0;
 	gdouble r = mmap_get_world_radius ();
+	gdouble near, far;
+
+	glMatrixMode (GL_PROJECTION);
+	glLoadIdentity ();
+
+	camera->fovy = 2.0 * DEG (atan ((height / 2.0) / l));
+	near = camera->dist * 0.2;
+	far = (r + camera->dist) * 2.0;
+	/*
+	   g_print("fovy:%.2f  near:%.2f  far:%.2f\n", camera->fovy, near, far);
+	 */
+
+	gluPerspective (camera->fovy, (gdouble) width / height, near, far);
 
 	glMatrixMode (GL_MODELVIEW);
 	glLoadIdentity ();
@@ -715,6 +744,18 @@ camera_get_dist (void)
 	return camera->dist;
 }
 
+gdouble
+camera_get_lon_target (void)
+{
+	return camera->lon_target;
+}
+
+gdouble
+camera_get_lat_target (void)
+{
+	return camera->lat_target;
+}
+
 gdouble
 camera_get_dist_target (void)
 {
@@ -725,17 +766,17 @@ camera_get_dist_target (void)
  * 左下(x0,y0) 右上(x1,y1) */
 /* 現在の場所の範囲でなく、目的の場所での範囲を返す */
 void
-camera_get_view (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1)
+camera_get_view_narrow (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1)
 {
 	gdouble west;
 	gdouble south;
 	gdouble east;
 	gdouble north;
 
-	west = camera->lon_target - camera->deg_view_lon / 2.0;
-	east = camera->lon_target + camera->deg_view_lon / 2.0;
-	north = camera->lat_target + camera->deg_view_lat / 2.0;
-	south = camera->lat_target - camera->deg_view_lat / 2.0;
+	west = camera->lon_target - camera->deg_view_lon_s / 2.0;
+	east = camera->lon_target + camera->deg_view_lon_s / 2.0;
+	north = camera->lat_target + camera->deg_view_lat_s / 2.0;
+	south = camera->lat_target - camera->deg_view_lat_s / 2.0;
 
 	if (west < -180.0) {
 		west += 360.0;
@@ -756,6 +797,40 @@ camera_get_view (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1)
 	*y1 = north;
 }
 
+void
+camera_get_view_wide (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1)
+{
+	gdouble west;
+	gdouble south;
+	gdouble east;
+	gdouble north;
+
+	west = camera->lon_target - camera->deg_view_lon_l / 2.0;
+	east = camera->lon_target + camera->deg_view_lon_l / 2.0;
+	north = camera->lat_target + camera->deg_view_lat_l / 2.0;
+	south = camera->lat_target - camera->deg_view_lat_l / 2.0;
+
+	if (west < -180.0) {
+		west += 360.0;
+	}
+	if (east > 180.0) {
+		east -= 360.0;
+	}
+	if (north > 90.0) {
+		north = 90.0;
+	}
+	if (south < -90.0) {
+		south = -90.0;
+	}
+
+	*x0 = west;
+	*y0 = south;
+	*x1 = east;
+	*y1 = north;
+}
+
+
+
 void
 camera_render_info (void)
 {
@@ -809,68 +884,103 @@ camera_render_info (void)
 void
 camera_debug_view_render (void)
 {
-	gdouble west;
-	gdouble south;
-	gdouble east;
-	gdouble north;
-	gdouble xyz[N_DEBUG][N_DEBUG][3];
+	gdouble west_l, east_l, south_l, north_l;
+	gdouble west_s, east_s, south_s, north_s;
+	gdouble xyz_l[N_DEBUG][N_DEBUG][3];
+	gdouble xyz_s[N_DEBUG][N_DEBUG][3];
 	gint i, j;
 	gdouble n = N_DEBUG;
 
-	west = camera->lon_target - camera->deg_view_lon / 2.0;
-	east = camera->lon_target + camera->deg_view_lon / 2.0;
-	north = camera->lat_target + camera->deg_view_lat / 2.0;
-	south = camera->lat_target - camera->deg_view_lat / 2.0;
+	west_l = camera->lon_target - camera->deg_view_lon_l / 2.0;
+	east_l = camera->lon_target + camera->deg_view_lon_l / 2.0;
+	north_l = camera->lat_target + camera->deg_view_lat_l / 2.0;
+	south_l = camera->lat_target - camera->deg_view_lat_l / 2.0;
+
+	west_s = camera->lon_target - camera->deg_view_lon_s / 2.0;
+	east_s = camera->lon_target + camera->deg_view_lon_s / 2.0;
+	north_s = camera->lat_target + camera->deg_view_lat_s / 2.0;
+	south_s = camera->lat_target - camera->deg_view_lat_s / 2.0;
 
 	for (j = 0; j < n; ++j) {
-		gdouble lat = south + (north - south) / (n - 1.0) * j;
+		gdouble lat_l = south_l + (north_l - south_l) / (n - 1.0) * j;
+		gdouble lat_s = south_s + (north_s - south_s) / (n - 1.0) * j;
 
 		for (i = 0; i < n; ++i) {
-			gdouble lon = west + (east - west) / (n - 1.0) * i;
-			gdouble x, y, z;
+			gdouble lon_l = west_l + (east_l - west_l) / (n - 1.0) * i;
+			gdouble lon_s = west_s + (east_s - west_s) / (n - 1.0) * i;
+			gdouble x_l, y_l, z_l;
+			gdouble x_s, y_s, z_s;
 
-			if (lon < -180.0) {
-				lon += 360.0;
+			if (lon_l < -180.0) {
+				lon_l += 360.0;
 			}
-			if (lon > 180.0) {
-				lon -= 360.0;
+			if (lon_l > 180.0) {
+				lon_l -= 360.0;
 			}
-			if (lat > 90.0) {
-				lat = 90.0;
+			if (lat_l > 90.0) {
+				lat_l = 90.0;
 			}
-			if (lat < -90.0) {
-				lat = -90.0;
+			if (lat_l < -90.0) {
+				lat_l = -90.0;
 			}
 
-			mmap_deg_to_xyz (lon, lat, 100.0, &x, &y, &z);
-			xyz[i][j][0] = x;
-			xyz[i][j][1] = y;
-			xyz[i][j][2] = z;
+			if (lon_s < -180.0) {
+				lon_s += 360.0;
+			}
+			if (lon_s > 180.0) {
+				lon_s -= 360.0;
+			}
+			if (lat_s > 90.0) {
+				lat_s = 90.0;
+			}
+			if (lat_s < -90.0) {
+				lat_s = -90.0;
+			}
+
+			mmap_deg_to_xyz (lon_l, lat_l, 100.0, &x_l, &y_l, &z_l);
+			mmap_deg_to_xyz (lon_s, lat_s, 100.0, &x_s, &y_s, &z_s);
+			xyz_l[i][j][0] = x_l;
+			xyz_l[i][j][1] = y_l;
+			xyz_l[i][j][2] = z_l;
+			xyz_s[i][j][0] = x_s;
+			xyz_s[i][j][1] = y_s;
+			xyz_s[i][j][2] = z_s;
 		}
 	}
 
 	{
 		gint i, j;
-		gdouble color[3] = { 1.0, 1.0, 1.0 };
+		gdouble color_l[3] = { 1.0, 1.0, 1.0 };
+		gdouble color_s[3] = { 0.5, 0.5, 0.5 };
 
 		glDisable (GL_DEPTH_TEST);
 		for (j = 0; j < n; ++j) {
 			for (i = 0; i < n - 1; ++i) {
-				glColor3dv (color);
 				glBegin (GL_LINE_STRIP);
-				glVertex3dv (xyz[i + 0][j]);
-				glColor3dv (color);
-				glVertex3dv (xyz[i + 1][j]);
-				glColor3dv (color);
+				glColor3dv (color_l);
+				glVertex3dv (xyz_l[i + 0][j]);
+				glVertex3dv (xyz_l[i + 1][j]);
+				glEnd ();
+
+				glBegin (GL_LINE_STRIP);
+				glColor3dv (color_s);
+				glVertex3dv (xyz_s[i + 0][j]);
+				glVertex3dv (xyz_s[i + 1][j]);
 				glEnd ();
 			}
 		}
 		for (j = 0; j < n; ++j) {
 			for (i = 0; i < n - 1; ++i) {
 				glBegin (GL_LINE_STRIP);
-				glColor3dv (color);
-				glVertex3dv (xyz[j][i + 0]);
-				glVertex3dv (xyz[j][i + 1]);
+				glColor3dv (color_l);
+				glVertex3dv (xyz_l[j][i + 0]);
+				glVertex3dv (xyz_l[j][i + 1]);
+				glEnd ();
+
+				glBegin (GL_LINE_STRIP);
+				glColor3dv (color_s);
+				glVertex3dv (xyz_s[j][i + 0]);
+				glVertex3dv (xyz_s[j][i + 1]);
 				glEnd ();
 			}
 		}
@@ -885,15 +995,13 @@ camera_check_overlap (gdouble north, gdouble south, gdouble west, gdouble east)
 	gdouble csouth;
 	gdouble ceast;
 	gdouble cnorth;
-
 	/*
 	   g_print("camera_check_overlap:n:%.2f s:%.2f w:%.2f e:%.2f\n", north, south, west, east);
 	 */
-
-	cwest = camera->lon - camera->deg_view_lon / 2.0;
-	ceast = camera->lon + camera->deg_view_lon / 2.0;
-	cnorth = camera->lat + camera->deg_view_lat / 2.0;
-	csouth = camera->lat - camera->deg_view_lat / 2.0;
+	cwest = camera->lon - camera->deg_view_lon_s / 2.0;
+	ceast = camera->lon + camera->deg_view_lon_s / 2.0;
+	cnorth = camera->lat + camera->deg_view_lat_s / 2.0;
+	csouth = camera->lat - camera->deg_view_lat_s / 2.0;
 
 	if (cwest < -180.0) {
 		cwest += 360.0;
@@ -907,11 +1015,9 @@ camera_check_overlap (gdouble north, gdouble south, gdouble west, gdouble east)
 	if (csouth < -90.0) {
 		csouth = -90.0;
 	}
-
 	/*
 	   g_print("camera_check_overlap:camera:n:%.2f s:%.2f w:%.2f e:%.2f\n", cnorth, csouth, cwest, ceast);
 	 */
-
 	return util_check_overlap (cwest, csouth, ceast, cnorth, west, south, east, north);
 }
 
@@ -932,3 +1038,15 @@ camera_get_inertia (void)
 {
 	return camera->inertia;
 }
+
+gdouble
+camera_get_fovy (void)
+{
+	return camera->fovy;
+}
+
+gdouble
+camera_get_deg_view_r (void)
+{
+	return camera->deg_view_r;
+}
