@@ -42,21 +42,25 @@ gdouble camera_get_lon (void);
 gdouble camera_get_lat (void);
 gdouble camera_get_alt (void);
 gdouble camera_get_dist (void);
+gdouble camera_get_lon_target (void);
+gdouble camera_get_lat_target (void);
 gdouble camera_get_dist_target (void);
 
 void camera_set_dist (gdouble dist);
+void camera_set_dist_direct (gdouble dist);
 void camera_dist_up (void);
 void camera_dist_down (void);
 
 void camera_reset (void);
 
-void camera_get_view (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1);
+void camera_get_view_narrow (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1);
+void camera_get_view_wide (gdouble * x0, gdouble * y0, gdouble * x1, gdouble * y1);
 
 void camera_set_position (gdouble lon, gdouble lat);
 void camera_debug_view_render (void);
 void camera_render_info (void);
 
-void camera_matrix_prepare (void);
+void camera_matrix_prepare (gint width, gint height);
 
 gboolean camera_check_overlap (gdouble north, gdouble south, gdouble west, gdouble east);
 
@@ -64,5 +68,7 @@ void camera_set_type (CameraType type);
 
 void camera_set_inertia (gboolean bool);
 gboolean camera_get_inertia (void);
+gdouble camera_get_fovy (void);
+gdouble camera_get_deg_view_r (void);
 
 #endif /* __CAMERA_H__ */
