@@ -33,9 +33,9 @@ typedef enum {
 GIOChannel *disk_channel_open (const gchar * fullpath, IOType type);
 
 GdkPixbuf *disk_load_jpg (const gchar * fullpath);
-gchar *disk_load_bin (const gchar * fullpath, gint * size);
 
-void disk_save_bin (const gchar * fullpath, gchar * data, gint size);
+void *disk_load_bin (const gchar * fullpath, gint * size);
+void disk_save_bin (const gchar * fullpath, void *data, gint size);
 
 gboolean filetest (const gchar * fullpath);
 
