@@ -125,9 +125,6 @@ typedef struct {
 
 static DragState ds;
 
-/* 視野の縦の角度 (度) */
-static gdouble fovy;
-
 typedef struct {
 	GtkWidget *glarea;
 
@@ -204,11 +201,11 @@ thread_func_update (GObject * obj)
 
 	if (n_update_thread == 1) {
 		/*
-		   g_print ("thread_func_update:%d:start\n", n_update_thread);
+		   g_print ("################# update start:%d\n", n_update_thread);
 		 */
 		mmap_update ();
 		/*
-		   g_print ("thread_func_update:%d:end\n", n_update_thread);
+		   g_print ("################# update finish:%d\n", n_update_thread);
 		 */
 	} else {
 	}
@@ -267,16 +264,15 @@ create_info (void)
 	if (n == 0) {
 		text_status[0] = '\0';
 	} else if (n == 1) {
-		sprintf (text_status, "Downloading ....  ( %d file )", n);
+		sprintf (text_status, "Downloading .... ( %d file )", n);
 	} else if (n > 1) {
-		sprintf (text_status, "Downloading ....  ( %d files )", n);
+		sprintf (text_status, "Downloading .... ( %d files )", n);
 	} else {
 		/* ありえん */
 		text_status[0] = '\0';
 	}
 
-	sprintf (text_info, "fps : %6.2f   render : %6.2f (msec)       %s", info_fps, info_render_time * 1000.0,
-			 text_status);
+	sprintf (text_info, "fps : %6.2f   render : %6.2f (msec)       %s", info_fps, info_render_time * 1000.0, text_status);
 }
 
 /*
@@ -334,7 +330,7 @@ glarea_saveimage (void)
 /******************************************/
 
 static guint idle_id;
-static gboolean idle_render = FALSE;
+static gboolean idle_render = FALSE;	/* idle ループに入った時に これが TRUE なら render() が走る */
 
 static void
 render_test (void)
@@ -380,39 +376,16 @@ cross_render (void)
 }
 
 static void
-set_projection_matrix (void)
+set_matrix (void)
 {
-	gdouble l = 1000.0;			/* 適当に選んだ値 */
-	gdouble near, far;
-	gdouble alt;
-	gdouble radius = mmap_get_world_radius ();
-
-	glMatrixMode (GL_PROJECTION);
-	glLoadIdentity ();
-
-	alt = camera_get_alt ();
-
-	fovy = 360.0 / G_PI * atan ((area->h / 2.0) / l);
-	near = alt * 0.2;
-	far = (radius + alt) * 2.0;
-	/*
-	   far = sqrt ((radius + alt) * (radius + alt) - radius * radius);
-	   g_print("fovy:%.2f  near:%.2f  far:%.2f\n", fovy, near, far);
-	 */
-	gluPerspective (fovy, (gdouble) area->w / area->h, near, far * 2.0);
-}
-
-static void
-set_modelview_matrix (void)
-{
-	camera_matrix_prepare ();
+	camera_matrix_prepare (area->w, area->h);
 }
 
 void
 glarea_render_file (void)
 {
-	set_projection_matrix ();
-	set_modelview_matrix ();
+	set_matrix ();
+
 	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
 	mmap_render ();
@@ -437,8 +410,7 @@ render (void)
 	{
 		glViewport (0, 0, area->w, area->h);
 		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-		set_projection_matrix ();
-		set_modelview_matrix ();
+		set_matrix ();
 
 		mmap_render ();
 
@@ -449,6 +421,7 @@ render (void)
 		render_test ();			/* 地球の粗い外形 */
 		camera_debug_view_render ();	/* 視野範囲 */
 #endif
+
 		camera_render_info ();	/* lon lat alt */
 		info_render (text_info, 1, 1);	/* fps & download info */
 
@@ -493,8 +466,10 @@ render_single (void)
 	GTimeVal stop_all;
 	gdouble sec_render;
 	gdouble sec_all;
-
-	idle_render = camera_next_step ();
+	/*
+	   g_print("render_single\n");
+	 */
+	idle_render = !(camera_next_step ());
 	create_info ();
 
 	g_get_current_time (&start);
@@ -518,17 +493,14 @@ static gboolean
 idle_func_render (gpointer data)
 {
 	gdouble sec_interval = 1.0 / 20.0;	/* 周期の目標（秒） */
+	/*  
+	   g_print ("idle_func_render\n");
+	 */
 
-	if (idle_render == FALSE) {
-		g_usleep ((gulong) sec_interval);
-
-	} else {
-		/*
-		   g_print ("idle_func_render\n");
-		 */
-
+	if (idle_render == TRUE) {
 		render_single ();
-
+	} else {
+		g_usleep ((gulong) sec_interval);
 	}
 	return TRUE;
 }
@@ -549,10 +521,6 @@ idle_func_render_single (gpointer data)
 	--render_single_request;
 	g_static_mutex_unlock (&mutex_render_single);
 
-	/*
-	   g_print ("idle_func_render_single:end\n");
-	 */
-
 	return FALSE;
 }
 
@@ -569,11 +537,16 @@ glarea_force_render (void)
 	g_static_mutex_unlock (&mutex_render_single);
 }
 
+void
+glarea_force_update (void)
+{
+	glarea_update ();
+}
+
 void
 glarea_force_update_and_render (void)
 {
 	glarea_update ();
-
 	glarea_force_render ();
 }
 
@@ -612,9 +585,10 @@ glarea_mouse_motion_cb (GtkWidget * glarea, GdkEventMotion * event, gpointer use
 				if (l != 0.0) {
 					gdouble dist = camera_get_dist ();
 					gdouble radius = mmap_get_world_radius ();
+					gdouble fovy = camera_get_fovy ();
 
 					/* ar:回転角度 */
-					double ar = l * 360.0 / 2.0 / G_PI * atan (dist / radius * tan (RAD (fovy / 2.0))) / 15000.0;
+					double ar = l * 360.0 / 2.0 / G_PI * atan (dist * tan (RAD (fovy / 2.0)) / radius) / 8000.0;
 					gdouble a = dy / l;
 					gdouble b = dx / l;
 
@@ -728,11 +702,11 @@ glarea_realize_cb (GtkWidget * glarea, gpointer user_data)
 	gldrawable_main = gtk_widget_get_gl_drawable (glarea);
 	is_direct = gdk_gl_context_is_direct (glcontext_main);
 	util_print_bool ("direct rendering enable ", is_direct);
-    if(is_direct ==FALSE){
-        g_print("############################################\n");
-        g_print("###     Direct Rendering ""DISABLE""     ###\n");
-        g_print("############################################\n");
-    }
+	if (is_direct == FALSE) {
+		g_print ("############################################\n");
+		g_print ("###     Direct Rendering " "DISABLE" "   ###\n");
+		g_print ("############################################\n");
+	}
 
 	if (!gdk_gl_drawable_gl_begin (gldrawable_main, glcontext_main)) {
 		g_print ("glarea_realize_cb:gl_begin:false\n");
@@ -741,7 +715,6 @@ glarea_realize_cb (GtkWidget * glarea, gpointer user_data)
 	{
 		gchar compress[] = "GL_ARB_texture_compression";
 		gchar vbo[] = "GL_ARB_vertex_buffer_object";
-		gint bytes;
 
 		glClearColor (0.4, 0.4, 0.4, 1.0);
 		glClearDepth (1.0);
@@ -752,9 +725,6 @@ glarea_realize_cb (GtkWidget * glarea, gpointer user_data)
 		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		/*
 		   glBlendFunc (GL_ONE, GL_ZERO);
-		 */
-
-		/*
 		   glEnable (GL_CULL_FACE);
 		   glCullFace (GL_FRONT);
 		 */
@@ -788,23 +758,51 @@ glarea_realize_cb (GtkWidget * glarea, gpointer user_data)
 		}
 
 		if (gdk_gl_query_gl_extension (compress) == TRUE) {
+			gint bytes;
+			gint num = 0;
+
 			g_print ("%s ------------ supported\n", compress);
-			support_compress = TRUE;
 
-			glGenTextures (1, &t_dummy);
-			glBindTexture (GL_TEXTURE_2D, t_dummy);
-			glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-			glTexImage2D (GL_TEXTURE_2D, 0, GL_COMPRESSED_RGB_ARB, 4, 4, 0, GL_RGB, GL_UNSIGNED_BYTE, dummy);
-			glGetTexLevelParameteriv (GL_TEXTURE_2D, 0, GL_TEXTURE_COMPRESSED_IMAGE_SIZE, &bytes);
-			/* こんな所で得た texture_format を全てのテクスチャーのフォーマットだとしてるが、いいのか？ */
-			glGetTexLevelParameteriv (GL_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &texture_format);
+			/* GL_ARB_texture_compression の拡張は、圧縮テクスチャーを利用するしくみを提供するだけで、
+			 * 実際の圧縮フォーマットの追加は、別の拡張にたよるようだ。
+			 * 圧縮フォーマットの数は、GL_NUM_COMPRESSED_TEXTURE_FORMATS で調べられるので、これがゼロ以上
+			 * でないとダメみたい。 */
 
-			g_print ("dummy:bytes:%d format:%x\n", bytes, texture_format);
+			glGetIntegerv (GL_NUM_COMPRESSED_TEXTURE_FORMATS, &num);
+			g_print ("num_compressed_texture_formats:%d\n", num);
+			if (num > 0) {
+				gboolean is_ok = TRUE;
+
+
+				glGenTextures (1, &t_dummy);
+				glBindTexture (GL_TEXTURE_2D, t_dummy);
+				glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+				glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+				glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+				glTexImage2D (GL_TEXTURE_2D, 0, GL_COMPRESSED_RGB_ARB, 4, 4, 0, GL_RGB, GL_UNSIGNED_BYTE, dummy);
+				is_ok = is_ok && gl_error ();
+				glGetTexLevelParameteriv (GL_TEXTURE_2D, 0, GL_TEXTURE_COMPRESSED_IMAGE_SIZE, &bytes);
+				is_ok = is_ok && gl_error ();
+				glGetTexLevelParameteriv (GL_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &texture_format);
+
+				g_print ("dummy:bytes:%d format:%x\n", bytes, texture_format);
+
+				if (is_ok == TRUE) {
+					g_print ("use compressed texture\n");
+					support_compress = TRUE;
+				} else {
+					/* ここにくることはあるのか？ */
+					g_print ("do not use compressed texture: gl_error\n");
+					support_compress = FALSE;
+				}
+			} else {
+				g_print ("not use compressed texture: no compressed texture format\n");
+				support_compress = FALSE;
+			}
 
 		} else {
 			g_print ("%s ------------ not supported\n", compress);
+			g_print ("not use compressed texture: extension not supported\n");
 			support_compress = FALSE;
 		}
 
@@ -860,8 +858,7 @@ glarea_create (void)
 	}
 
 	glarea = gtk_drawing_area_new ();
-	gtk_widget_set_events (glarea, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK |
-						   GDK_BUTTON_PRESS_MASK | GDK_EXPOSURE_MASK);
+	gtk_widget_set_events (glarea, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_EXPOSURE_MASK);
 	if (gtk_widget_set_gl_capability (glarea, glconfig, NULL, TRUE, GDK_GL_RGBA_TYPE) == FALSE) {
 		g_print ("glarea_create:gtk_widget_set_gl_capability:fail\n");
 		exit (1);
