@@ -35,17 +35,26 @@
 #include "color.h"
 #include "gsi.h"
 
+/* (px0, py0) - (px1, py1) の範囲を (0, 0)-(1, 1) とした時の
+ * (cx0, cy0) - (cx1, cy1) の範囲のメッシュを作って返す。 */
 gdouble *
-mesh_create_st (gint nx, gint ny, WwMeshType type)
+mesh_create_st (gdouble px0, gdouble py0, gdouble px1, gdouble py1, gdouble cx0, gdouble cy0, gdouble cx1, gdouble cy1,
+				gint nx, gint ny)
 {
 	gint i, j;
 	gint c;
 	gdouble *st;
-
+	gdouble x0, y0, x1, y1;		/* (0.2, 0.2)-(0.3, 0.5) とか */
 	/*
-	   g_print ("mesh_create_st:%d\n", type);
+	   g_print ("mesh_create_st:p:%.2f %.2f %.2f %.2f\n", px0, py0, px1, py1);
+	   g_print ("               c:%.2f %.2f %.2f %.2f\n", cx0, cy0, cx1, cy1);
 	 */
 
+	x0 = (cx0 - px0) / (px1 - px0);
+	y0 = (cy0 - py0) / (py1 - py0);
+	x1 = (cx1 - px0) / (px1 - px0);
+	y1 = (cy1 - py0) / (py1 - py0);
+
 	st = g_new (gdouble, (nx + 1) * (ny + 1) * 2);
 
 	c = 0;
@@ -53,27 +62,9 @@ mesh_create_st (gint nx, gint ny, WwMeshType type)
 		for (i = 0; i < nx + 1; ++i) {
 			gdouble s, t;
 
-			s = (gdouble) i / nx;
-			t = 1.0 - (gdouble) j / ny;
+			s = x0 + (x1 - x0) * (gdouble) i / nx;
+			t = 1.0 - (y0 + (y1 - y0) * (gdouble) j / ny);
 
-			if (type == WW_MESH_TYPE_NORMAL) {
-				/*
-				   s = s;
-				   t = t;
-				 */
-			} else if (type == WW_MESH_TYPE_NORTH_WEST) {
-				s = s / 2.0;
-				t = t / 2.0;
-			} else if (type == WW_MESH_TYPE_NORTH_EAST) {
-				s = s / 2.0 + 0.5;
-				t = t / 2.0;
-			} else if (type == WW_MESH_TYPE_SOUTH_WEST) {
-				s = s / 2.0;
-				t = t / 2.0 + 0.5;
-			} else if (type == WW_MESH_TYPE_SOUTH_EAST) {
-				s = s / 2.0 + 0.5;
-				t = t / 2.0 + 0.5;
-			}
 
 			*(st + 2 * c + 0) = s;
 			*(st + 2 * c + 1) = t;
@@ -88,6 +79,12 @@ mesh_create_st (gint nx, gint ny, WwMeshType type)
 	return st;
 }
 
+gdouble *
+mesh_create_st_simple (gint nx, gint ny)
+{
+	return mesh_create_st (0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0, nx, ny);
+}
+
 /* ****************
  * 現在、経度が180度をまたぐ場合は使えない。
  * (x0,y0):左下
@@ -95,98 +92,52 @@ mesh_create_st (gint nx, gint ny, WwMeshType type)
  * ***********************要変更******************:
  */
 
-/* 全部標高ゼロ版 */
+/* 親の標高データを使って作る。*/
 gdouble *
-mesh_create_deg_alt0 (gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, gint ny)
+mesh_create_deg_copy (gdouble px0, gdouble py0, gdouble px1, gdouble py1, gdouble cx0, gdouble cy0, gdouble cx1, gdouble cy1,
+					  gint nx, gint ny, gdouble * parent)
 {
 	gint i, j;
 	gint c;
 	gdouble *v;
 	gdouble dx, dy;
+	gdouble x0, y0, x1, y1;		/* (0.2, 0.2)-(0.3, 0.5) とか */
+	/*
+	   g_print ("mesh_create_deg_copy:p:%.2f %.2f %.2f %.2f  c:%.2f %.2f %.2f %.2f\n", px0, py0, px1, py1, cx0, cy0, cx1, cy1);
+	 */
 
-	dx = (x1 - x0) / nx;
-	dy = (y1 - y0) / ny;
+	dx = (cx1 - cx0) / nx;
+	dy = (cy1 - cy0) / ny;
+
+	x0 = (cx0 - px0) / (px1 - px0);
+	y0 = (cy0 - py0) / (py1 - py0);
+	x1 = (cx1 - px0) / (px1 - px0);
+	y1 = (cy1 - py0) / (py1 - py0);
 
 	v = g_new (gdouble, (nx + 1) * (ny + 1) * 3);
 
 	c = 0;
 	for (j = 0; j < ny + 1; ++j) {
 		for (i = 0; i < nx + 1; ++i) {
-			*(v + 3 * c + 0) = x0 + i * dx;
-			*(v + 3 * c + 1) = y0 + j * dy;
-			*(v + 3 * c + 2) = 0.0;
-			++c;
-		}
-	}
-
-	return v;
-}
-
-/* 親の標高データを使って作る。４点を同じ標高にする */
-gdouble *
-mesh_create_deg_copy (gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, gint ny, WwMeshType type,
-					  gdouble * v_deg_parent)
-{
-	gint i, j;
-	gint c;
-	gdouble *v;
-	gdouble dx, dy;
-	gint x_offset = 0;
-	gint y_offset = 0;
-
-	if (type == WW_MESH_TYPE_NORMAL) {
-		g_print ("mesh_create_deg_copy:WW_MESH_TYPE_NORMAL ??????\n");	/* ありえない */
-		exit (1);
-	} else if (type == WW_MESH_TYPE_NORTH_WEST) {
-		y_offset = ny / 2;
-	} else if (type == WW_MESH_TYPE_NORTH_EAST) {
-		x_offset = nx / 2;
-		y_offset = ny / 2;
-	} else if (type == WW_MESH_TYPE_SOUTH_WEST) {
-		;
-	} else if (type == WW_MESH_TYPE_SOUTH_EAST) {
-		x_offset = nx / 2;
-	}
-
-	dx = (x1 - x0) / nx;
-	dy = (y1 - y0) / ny;
-
-	v = g_new (gdouble, (nx + 1) * (ny + 1) * 3);
-
-#if 0
-	c = 0;
-	for (j = 0; j < ny + 1; ++j) {
-		g_print ("p(%d):", j);
-
-		for (i = 0; i < nx + 1; ++i) {
-			g_print ("%.0f ", *(v_deg_parent + 3 * c + 2));
-
-			++c;
-		}
-		g_print ("\n");
-	}
-
-	g_print ("create_v_deg_copy:type %d   x_off:%d y_off:%d   \n", type, x_offset, y_offset);
-#endif
-
-	c = 0;
-	for (j = 0; j < ny + 1; ++j) {
-		for (i = 0; i < nx + 1; ++i) {
+			gdouble id, jd;
 			gint ii, jj;
 			gint k;
 
-			ii = i / 2 + x_offset;
-			jj = j / 2 + y_offset;
+			id = x0 + (x1 - x0) * (gdouble) i / nx;
+			jd = y0 + (y1 - y0) * (gdouble) j / ny;
+
+			ii = id * nx;
+			jj = jd * ny;
 
 			k = ii + (nx + 1) * jj;
-
 			/*
-			   g_print ("  ii:%d jj:%d k:%d   %.2f\n", ii, jj, k, *(v_deg_parent + 3 * k + 2));
+			   g_print ("  ii:%d jj:%d k:%d   %.2f\n", ii, jj, k, *(parent + 3 * k + 2));
 			 */
 
-			*(v + 3 * c + 0) = x0 + i * dx;
-			*(v + 3 * c + 1) = y0 + j * dy;
-			*(v + 3 * c + 2) = *(v_deg_parent + 3 * k + 2);
+			*(v + 3 * c + 0) = cx0 + i * dx;
+			*(v + 3 * c + 1) = cy0 + j * dy;
+			*(v + 3 * c + 2) = *(parent + 3 * k + 2);
+
 			++c;
 		}
 	}
@@ -194,7 +145,7 @@ mesh_create_deg_copy (gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, g
 	return v;
 }
 
-/* 標高を terrain_tile から取る版 */
+/* 標高を terrain_tile から取る。terrainaccessor が NULL なら標高は全部ゼロ。  */
 gdouble *
 mesh_create_deg (gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, gint ny, WwTerrainaccessor * ta)
 {
@@ -219,7 +170,12 @@ mesh_create_deg (gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, gint n
 
 			*(v + 3 * c + 0) = x;
 			*(v + 3 * c + 1) = y;
-			*(v + 3 * c + 2) = ww_terrainaccessor_get_alt (ta, x, y);
+			if (ta == NULL) {
+				*(v + 3 * c + 2) = 0.0;
+			} else {
+				*(v + 3 * c + 2) = ww_terrainaccessor_get_alt (ta, x, y);
+			}
+
 			/*
 			   g_print("mesh_create_deg:%.2f %.2f ::: %.2f\n", x, y, *(v + 3 * c + 2));
 			 */
@@ -365,7 +321,7 @@ mesh_mmap_v_deg_init (gdouble x, gdouble y, gint nx, gint ny)
 	x1 = (x + W_SECOND + 1.0) / 3600.0;
 	y1 = (y + H_SECOND) / 3600.0;
 
-	return mesh_create_deg_alt0 (x0, y0, x1, y1, nx, ny);
+	return mesh_create_deg (x0, y0, x1, y1, nx, ny, NULL);
 }
 
 void
@@ -502,7 +458,7 @@ mesh_mmap_init (void)
 	nt_mmap = (nx + 1) * (ny + 1);
 	ni_mmap = ((nx + 1) * 2 + 2) * ny;
 
-	t_st_mmap = mesh_create_st (nx, ny, WW_MESH_TYPE_NORMAL);
+	t_st_mmap = mesh_create_st_simple (nx, ny);
 	index_mmap = mesh_create_index (nx, ny);
 
 }
