@@ -388,8 +388,7 @@ lzh_extract (const gchar * lzh)
 	 */
 	current_dir = g_get_current_dir ();
 	chdir (mmap_dir_gsi_vn);
-	g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL, NULL, NULL, NULL, NULL, NULL,
-				  &err);
+	g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL, NULL, NULL, NULL, NULL, NULL, &err);
 	if (err != NULL) {
 		g_print ("spawn error::lzh_extract:%s:\n", err->message);
 	}
@@ -451,14 +450,13 @@ is_far_enough (City * city)
 	gdouble far_enough = 100.0;	/* 200秒(5km位はあるか？)以上遠いと十分遠いと判断する */
 	/* マップ単位でしか比較してないので 100 にしても 150 とか 225 でしかかわらない */
 
-	camera_get_view (&x0, &y0, &x1, &y1);
+	camera_get_view_narrow (&x0, &y0, &x1, &y1);
 	x0 *= 3600.0;
 	y0 *= 3600.0;
 	x1 *= 3600.0;
 	y1 *= 3600.0;
 
-	if (x0 - city->x1 > far_enough || city->x0 - x1 > far_enough ||
-		y0 - city->y1 > far_enough || city->y0 - y1 > far_enough) {
+	if (x0 - city->x1 > far_enough || city->x0 - x1 > far_enough || y0 - city->y1 > far_enough || city->y0 - y1 > far_enough) {
 
 		/*
 		   g_print ("is_far_enough:TRUE\n");
