@@ -46,12 +46,8 @@ typedef struct {
 
 static GSList *sl_downloadthreadpool = NULL;	/* 中身は DownloadThreadPool */
 
-
-
-
 static gpointer thread_func_download (gpointer data, gpointer user_data);
 
-
 static gchar *
 get_key_from_uri (const gchar * uri)
 {
@@ -136,7 +132,7 @@ get_downloadpool (const gchar * uri)
 
 		sl_downloadthreadpool = g_slist_prepend (sl_downloadthreadpool, dt_new);
 		n = g_slist_length (sl_downloadthreadpool);
-		g_print ("################################################################ %d\n", n);
+		g_print ("################################################# %d:%s\n", n, key);
 
 
 		dpool = dt_new;
@@ -183,7 +179,6 @@ static gpointer
 thread_func_download (gpointer data, gpointer user_data)
 {
 	ThreadData *td = data;
-	GObject *obj = G_OBJECT (td->obj);
 
 	g_mutex_lock (td->pool->mutex);
 	if (td->pool->curl == NULL) {
@@ -191,7 +186,14 @@ thread_func_download (gpointer data, gpointer user_data)
 	}
 	g_mutex_unlock (td->pool->mutex);
 
-	td->func (obj);
+	if (td->obj->ref_count == 1) {
+		/*
+		   g_print ("thread_func_download  detatched:%p:\n", td->obj);
+		 */
+		;
+	} else {
+		td->func (td->obj);
+	}
 
 	/* これをしないことで keep-alive  ?
 	   g_mutex_lock(td->pool->mutex);
@@ -200,11 +202,11 @@ thread_func_download (gpointer data, gpointer user_data)
 	   g_mutex_unlock(td->pool->mutex);
 	 */
 
+	td->pool->n -= 1;
 
-	g_object_unref (obj);
+	g_object_unref (td->obj);
 	g_free (td);
 
-	td->pool->n -= 1;
 	--n_download;
 
 	return NULL;
@@ -215,15 +217,18 @@ thread_func_power (gpointer data, gpointer user_data)
 {
 	ThreadData *td = data;
 
-	if (td->obj == NULL) {
-		td->func (NULL);
-
+	if (td->obj != NULL && td->obj->ref_count == 1) {
+		/*
+		   g_print ("thread_func_power  detatched:%p:\n", td->obj);
+		 */
+		;
 	} else {
-		GObject *obj = G_OBJECT (td->obj);
-		td->func (obj);
-		g_object_unref (obj);
+		td->func (td->obj);
 	}
 
+	if (td->obj != NULL) {
+		g_object_unref (td->obj);
+	}
 	g_free (td);
 
 	--n_power;
@@ -237,14 +242,14 @@ mmapthread_add_power (MmapThreadFunc func, GObject * obj)
 	ThreadData *td;
 	GError *err = NULL;
 
-	if (obj != NULL) {
-		g_object_ref (obj);
-	}
-
 	/*
 	   g_print ("mmapthread_add_power:%d\n",g_thread_pool_unprocessed(pool_power));
 	 */
 
+	if (obj != NULL) {
+		g_object_ref (obj);
+	}
+
 	td = g_new (ThreadData, 1);
 	td->func = func;
 	td->obj = obj;
@@ -260,8 +265,6 @@ mmapthread_add_power (MmapThreadFunc func, GObject * obj)
 
 /************************/
 
-
-
 void
 mmapthread_add_download2 (const gchar * uri, MmapThreadFunc func, GObject * obj)
 {
