@@ -139,20 +139,6 @@ util_get_l (gdouble x0, gdouble y0, gdouble z0, gdouble x1, gdouble y1, gdouble
 	return l;
 }
 
-gdouble
-util_get_degree_from_lonlat (gdouble lon0, gdouble lat0, gdouble lon1, gdouble lat1)
-{
-	gdouble deg;
-	gdouble lon0_rad = 2.0 * G_PI * lon0 / 360.0;
-	gdouble lat0_rad = 2.0 * G_PI * lat0 / 360.0;
-	gdouble lon1_rad = 2.0 * G_PI * lon1 / 360.0;
-	gdouble lat1_rad = 2.0 * G_PI * lat1 / 360.0;
-
-	deg = acos (cos (lat0_rad) * cos (lat1_rad) * cos (lon1_rad - lon0_rad) + sin (lat0_rad) * sin (lat1_rad));
-
-	return deg;
-}
-
 void
 util_mkdir_p (const gchar * fullpath)
 {
@@ -363,15 +349,19 @@ util_tile_get_view (gdouble west, gdouble south, gdouble east, gdouble north, gd
 	}
 }
 
-void
+gboolean
 gl_error (void)
 {
 	GLenum err = glGetError ();
 
-	if (err != GL_NO_ERROR) {
-		g_print ("gl_err:%s\n", gluErrorString (err));
+	if (err == GL_NO_ERROR) {
+		g_print ("gl_error:no error\n");
+
+		return TRUE;
 	} else {
-		g_print ("gl_err:no error\n");
+		g_print ("gl_error:%s\n", gluErrorString (err));
+
+		return FALSE;
 	}
 }
 
@@ -454,8 +444,7 @@ util_pixbuf_scale_down (const GdkPixbuf * buf, gboolean left, gint size)
 	buf_new = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, size, size);
 
 	DB (g_print ("before_scale\n"));
-	gdk_pixbuf_scale (buf, buf_new, 0, 0, size, size, -w_left * scale_x, -h_top * scale_y,
-					  scale_x, scale_y, GDK_INTERP_BILINEAR);
+	gdk_pixbuf_scale (buf, buf_new, 0, 0, size, size, -w_left * scale_x, -h_top * scale_y, scale_x, scale_y, GDK_INTERP_BILINEAR);
 	DB (g_print ("after_scale\n"));
 
 	return buf_new;
@@ -498,3 +487,11 @@ util_get_cache_size (void)
 
 	return size;
 }
+
+/*********************************/
+
+void
+util_print_ref_count (const gchar * text, GObject * obj)
+{
+	g_print (text, obj->ref_count);
+}
