@@ -117,8 +117,7 @@ open_world_action (GtkAction * action)
 	GtkWidget *dialog;
 
 	dialog = gtk_file_chooser_dialog_new (_("Open World"), GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_OPEN,
-										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-										  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), mmap_dir_pkg_config);
 
@@ -487,8 +486,7 @@ window_create (void)
 	gtk_action_group_add_toggle_actions (action_group, toggle_entries, n_toggle_entries, NULL);
 	gtk_action_group_add_radio_actions (action_group, camera_entries, n_camera_entries, RADIO_CAMERA_TYPE_NWW,
 										G_CALLBACK (camera_type_action), NULL);
-	gtk_action_group_add_radio_actions (action_group, ve_entries, n_ve_entries, RADIO_VE_X010, G_CALLBACK (ve_action),
-										NULL);
+	gtk_action_group_add_radio_actions (action_group, ve_entries, n_ve_entries, RADIO_VE_X010, G_CALLBACK (ve_action), NULL);
 
 	ui_manager = gtk_ui_manager_new ();
 
