@@ -33,13 +33,14 @@
 #include "ww_object.h"
 #include "ww_image.h"
 #include "ww_texture.h"
+#include "ww_textureloader.h"
 #include "ww_mesh.h"
+#include "ww_meshst.h"
 #include "util.h"
 #include "ww_layerset.h"
+#include "mmapthread.h"
 #include "camera.h"
 #include "glarea.h"
-#include "mesh.h"
-#include "disk.h"
 
 typedef struct _WwImagePrivate WwImagePrivate;
 struct _WwImagePrivate {
@@ -66,9 +67,13 @@ struct _WwImagePrivate {
 
 	/*****/
 
-
 	WwTexture *texture;
+	WwMeshst *meshst;
 	WwMesh *mesh;
+	WwTextureloader *textureloader;
+
+	gchar *path_jpg;
+	gchar *path_dds;
 };
 
 #define WW_IMAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), WW_TYPE_IMAGE, WwImagePrivate))
@@ -124,9 +129,9 @@ ww_image_get_type (void)
 
 		g_type_add_interface_static (type, WW_TYPE_DATA, &ww_data_info);
 
-        /*
-		g_print ("ww_image_get_type\n");
-        */
+		/*
+		   g_print ("ww_image_get_type\n");
+		 */
 	}
 
 	return type;
@@ -137,11 +142,9 @@ ww_image_class_init (WwImageClass * klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	WwObjectClass *wwobject_class = WW_OBJECT_CLASS (klass);
-
 	/*
 	   g_print ("ww_image_class_init:klass:%p:\n", klass);
 	 */
-
 	g_type_class_add_private (klass, sizeof (WwImagePrivate));
 
 	parent_class = g_type_class_peek_parent (klass);
@@ -159,11 +162,9 @@ static void
 ww_image_init (WwImage * self)
 {
 	WwImagePrivate *priv = WW_IMAGE_GET_PRIVATE (self);
-
 	/*
 	   g_print ("ww_image_init:self:%p:\n", self);
 	 */
-
 	priv->dispose_has_run = FALSE;
 	priv->distance_above_surface = 0.0;
 	priv->north = 0.0;
@@ -178,9 +179,12 @@ ww_image_init (WwImage * self)
 	priv->transparency_color_red = 0;
 	priv->legend_image_path = NULL;
 
-
 	priv->texture = NULL;
+	priv->textureloader = NULL;
+	priv->meshst = NULL;
 	priv->mesh = NULL;
+	priv->path_jpg = 0;
+	priv->path_dds = 0;
 }
 
 static void
@@ -196,13 +200,25 @@ ww_image_dispose (GObject * obj)
 	if (priv->dispose_has_run) {
 		return;
 	}
+
 	priv->dispose_has_run = TRUE;
 
-	g_object_unref (priv->texture);
-	priv->texture = NULL;
-
-	g_object_unref (priv->mesh);
-	priv->mesh = NULL;
+	if (priv->texture != NULL) {
+		g_object_unref (priv->texture);
+		priv->texture = NULL;
+	}
+	if (priv->textureloader != NULL) {
+		g_object_unref (priv->textureloader);
+		priv->textureloader = NULL;
+	}
+	if (priv->meshst != NULL) {
+		g_object_unref (priv->meshst);
+		priv->meshst = NULL;
+	}
+	if (priv->mesh != NULL) {
+		g_object_unref (priv->mesh);
+		priv->mesh = NULL;
+	}
 
 	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
@@ -212,13 +228,17 @@ ww_image_finalize (GObject * obj)
 {
 	WwImage *self = WW_IMAGE (obj);
 	WwImagePrivate *priv = WW_IMAGE_GET_PRIVATE (self);
-
 	/*
 	   g_print ("finalize\n");
 	 */
-
 	g_free (priv->texture_path);
 	g_free (priv->legend_image_path);
+	if (priv->path_jpg != NULL) {
+		g_free (priv->path_jpg);
+	}
+	if (priv->path_dds != NULL) {
+		g_free (priv->path_dds);
+	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
@@ -228,11 +248,9 @@ ww_image_constructor (GType type, guint n_props, GObjectConstructParam * props)
 {
 	GObject *object;
 	GObjectClass *object_class = G_OBJECT_CLASS (parent_class);
-
 	/*
 	   g_print ("constructor\n");
 	 */
-
 	object = object_class->constructor (type, n_props, props);
 
 	return object;
@@ -261,25 +279,6 @@ ww_image_new (void)
 
 /********************************************************************/
 
-static void
-image_change_texture_cb (WwTexture * texture, gpointer data)
-{
-	WwImagePrivate *priv;
-	WwImage *image;
-
-	g_return_if_fail (WW_IS_TEXTURE (texture));
-	g_return_if_fail (WW_IS_IMAGE (data));
-
-	image = WW_IMAGE (data);
-
-	priv = WW_IMAGE_GET_PRIVATE (image);
-
-	g_object_unref (image);
-
-
-	priv->texture = texture;
-}
-
 static void
 ww_image_update (WwObject * obj)
 {
@@ -302,53 +301,75 @@ ww_image_update (WwObject * obj)
 	 */
 
 	if (priv->texture == NULL) {
-		gchar *path;
-		gchar *path_dds;
-		gchar *path_jpg;
-		gint w, h;
-		WwTexture *texture = NULL;
+		if (priv->textureloader == NULL) {
+			const gchar *world_name;
+			gchar *path;
+			gchar *filename_jpg;
+			gchar *filename_dds;
 
-		path = ww_object_get_path (obj);
-		if (g_strrstr (priv->texture_path, "land_shallow_topo_2048") != NULL) {
-			path_dds = g_strconcat (mmap_dir, priv->texture_path, NULL);
-			path_jpg = g_strconcat (mmap_dir_pkg, "/", "Data/Earth/", "land_shallow_topo_2048.jpg", NULL);
-			w = 2048;
-			h = 1024;
-		} else {
-			path_dds = NULL;
-			path_jpg = g_strconcat (mmap_dir, priv->texture_path, NULL);
-			w = 512;
-			h = 512;
-		}
+			world_name = mmap_get_world_name ();
+			path = ww_object_get_path (obj);
+			filename_jpg = g_path_get_basename (priv->texture_path);
+			{
+				gchar *p;
+				gchar *tmp;
 
-		g_print ("ww_image_update:priv->texture_path:%s\n", priv->texture_path);
-		g_print ("ww_image_update:path_dds:%s  path_jpg:%s\n", path_dds, path_jpg);
+				p = g_strrstr (filename_jpg, ".");
+				tmp = g_strndup (filename_jpg, p - filename_jpg);
+				filename_dds = g_strconcat (tmp, ".dds", NULL);
 
-		if (glarea_is_support_compress () == TRUE) {
-			if (filetest (path_dds) == TRUE) {
-				texture = ww_texture_new_dds (path_dds, w, h);
-			} else if (filetest (path_jpg) == TRUE) {
-				texture = ww_texture_new_jpg (path_jpg, path_dds);
+				g_free (tmp);
+			}
+			priv->path_jpg = g_strdup_printf ("%s%s/%s", mmap_dir_cache, world_name, filename_jpg);
+			priv->path_dds = g_strdup_printf ("%s%s/%s", mmap_dir_cache, world_name, filename_dds);
+
+			g_print ("ww_image_update:priv->texture_path:%s\n", priv->texture_path);
+			g_print ("ww_image_update:path_dds:%s\n", priv->path_dds);
+			g_print ("ww_image_update:path_jpg:%s\n", priv->path_jpg);
+
+			/*  Base ᡼ˤȤäƤʤΤǥ¾β㤨βˤ
+			 * Ĥȡꡣ׽*/
+			priv->textureloader = ww_textureloader_lookup (priv->texture_path);
+			if (priv->textureloader == NULL) {
+				WwTexture *texture = ww_texture_new ();
+
+				priv->textureloader = ww_textureloader_new ();
+				ww_textureloader_set_texture (priv->textureloader, texture);
+				ww_textureloader_set_source (priv->textureloader, priv->path_dds, priv->path_jpg, priv->texture_path, 2048, 1024);
+				ww_textureloader_start (priv->textureloader);
+			} else {
+				/* ˥椫ɤ߹ */
+				;
 			}
 
+			g_free (path);
+			g_free (filename_dds);
+			g_free (filename_jpg);
+
 		} else {
-			texture = ww_texture_new_jpg (path_jpg, path_dds);
+			if (priv->texture == NULL && priv->textureloader != NULL) {
+				priv->texture = ww_textureloader_get_texture (priv->textureloader);
+
+				if (priv->texture != NULL) {
+					g_object_ref (priv->texture);
+
+					g_object_unref (priv->textureloader);
+					priv->textureloader = NULL;
+
+				}
+				glarea_force_render ();
+			}
 		}
-		g_object_ref (obj);
-		g_signal_connect (G_OBJECT (texture), "initialized", G_CALLBACK (image_change_texture_cb), obj);
-		mmap_idle_add_single (ww_texture_gen, G_OBJECT (texture));
-
-		g_free (path);
-		g_free (path_dds);
-		g_free (path_jpg);
-
-		glarea_force_render ();
 	}
 
+	if (priv->meshst == NULL) {
+		gint split = 64;
+		priv->meshst = ww_meshst_new_simple (split, split / 2);
+	}
 	if (priv->mesh == NULL) {
 		gint split = 64;
-		priv->mesh = ww_mesh_new_simple (priv->east, priv->west, priv->north, priv->south,
-										 split, split / 2, priv->distance_above_surface, priv->terrain_mapped);
+		priv->mesh = ww_mesh_new_with_terrain (priv->west, priv->south, priv->east, priv->north,
+											   split, split / 2, priv->distance_above_surface, NULL);
 	}
 	obj->visible = TRUE;
 }
@@ -364,10 +385,12 @@ ww_image_render (WwObject * self)
 	/*
 	   g_print ("ww_image_render:%s\n", self->name);
 	 */
-	if (ww_texture_bind (priv->texture) == TRUE) {
-		ww_mesh_render (priv->mesh, WW_MESH_RENDER_TYPE_TEXTURE, FALSE, 0);
-	} else {
+	if (priv->texture == NULL) {
 		;
+	} else {
+		ww_texture_bind (priv->texture);
+		ww_meshst_bind (priv->meshst);
+		ww_mesh_render (priv->mesh, WW_MESH_RENDER_TYPE_TEXTURE, FALSE, 0);
 	}
 }
 
@@ -375,12 +398,11 @@ static void
 ww_image_set_on (WwObject * obj, gboolean is_on)
 {
 	WwObject *obj_parent = obj->wwparent;
-
-	g_print ("ww_image_set_on:name:%s:\n", obj->name);
-
+	/*
+	   g_print ("ww_image_set_on:name:%s:\n", obj->name);
+	 */
 	if (is_on == TRUE && WW_IS_LAYERSET (obj_parent)) {
 		/* is_on == TRUE λΤߤǤʤȡ롼פ */
-		g_print ("err3\n");
 		ww_layerset_check_show_only_one_layer (WW_LAYERSET (obj_parent));
 	}
 
@@ -427,7 +449,9 @@ ww_image_set_element (WwData * data, const gchar * element0, const gchar * eleme
 
 	} else if (g_ascii_strcasecmp (element0, "TexturePath") == 0) {
 		gchar *tmp = g_strdup (value);
-		util_separator_dos_to_unix (tmp);
+		/* ˤ http://.... ȤͥåȤΥɥ쥹񤤤ƤΤǺ
+		   util_separator_dos_to_unix (tmp);
+		 */
 		priv->texture_path = tmp;
 
 	} else if (g_ascii_strcasecmp (element0, "Opacity") == 0) {
