@@ -51,9 +51,6 @@ struct _WwMeshPrivate {
 	gdouble *v_deg;
 	gdouble *v_xyz;
 	guint *index;
-
-	gdouble *t_st;
-	WwMeshType type;
 };
 
 #define WW_MESH_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), WW_TYPE_MESH, WwMeshPrivate))
@@ -66,9 +63,6 @@ static void ww_mesh_finalize (GObject * object);
 static void ww_mesh_dispose (GObject * object);
 static GObject *ww_mesh_constructor (GType type, guint n_props, GObjectConstructParam * props);
 
-/*****************************************************************************/
-
-
 /*****************************************************************************/
 
 GType
@@ -90,9 +84,6 @@ ww_mesh_get_type (void)
 		};
 
 		type = g_type_register_static (G_TYPE_OBJECT, "WwMesh", &info, 0);
-        /*
-		g_print ("ww_mesh_get_type\n");
-        */
 	}
 
 	return type;
@@ -133,8 +124,6 @@ ww_mesh_init (WwMesh * self)
 	priv->v_deg = NULL;
 	priv->v_xyz = NULL;
 	priv->index = NULL;
-	priv->t_st = NULL;
-	priv->type = WW_MESH_TYPE_NORMAL;
 }
 
 static void
@@ -142,11 +131,9 @@ ww_mesh_dispose (GObject * obj)
 {
 	WwMesh *self = WW_MESH (obj);
 	WwMeshPrivate *priv = WW_MESH_GET_PRIVATE (self);
-
 	/*
 	   g_print ("ww_mesh_dispose\n");
 	 */
-
 	if (priv->dispose_has_run) {
 		return;
 	}
@@ -166,7 +153,6 @@ ww_mesh_finalize (GObject * obj)
 	g_free (priv->v_deg);
 	g_free (priv->v_xyz);
 	g_free (priv->index);
-	g_free (priv->t_st);
 
 	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
@@ -199,43 +185,7 @@ ww_mesh_new (void)
 /********************************************************************/
 
 WwMesh *
-ww_mesh_new_simple (gdouble lon_max, gdouble lon_min, gdouble lat_max, gdouble lat_min,
-					gint nx, gint ny, gdouble above_surface, gboolean terrain_mapped)
-{
-	WwMesh *mesh;
-	WwMeshPrivate *priv;
-	gdouble multi;
-	/*
-	   g_print ("ww_mesh_new_simple:\n");
-	 */
-	mesh = ww_mesh_new ();
-	priv = WW_MESH_GET_PRIVATE (mesh);
-
-	priv->distance_above_surface = above_surface;
-	priv->terrain_mapped = terrain_mapped;
-	priv->nx = nx;
-	priv->ny = ny;
-	priv->nv = (nx + 1) * (ny + 1);
-	priv->ni = ((nx + 1) * 2 + 2) * ny;
-
-	if (terrain_mapped == TRUE) {
-		multi = mmap_get_vertical_exaggeration ();
-	} else {
-		multi = 1.0;
-	}
-
-	priv->v_deg = mesh_create_deg_alt0 (lon_min, lat_min, lon_max, lat_max, nx, ny);
-	priv->v_xyz = mesh_create_xyz (priv->v_deg, nx, ny, above_surface, multi);
-	priv->index = mesh_create_index (nx, ny);
-
-	priv->t_st = mesh_create_st (nx, ny, WW_MESH_TYPE_NORMAL);
-	priv->type = WW_MESH_TYPE_NORMAL;
-
-	return mesh;
-};
-
-WwMesh *
-ww_mesh_new_with_terrain (gdouble lon_max, gdouble lon_min, gdouble lat_max, gdouble lat_min,
+ww_mesh_new_with_terrain (gdouble x0, gdouble y0, gdouble x1, gdouble y1,
 						  gint nx, gint ny, gdouble above_surface, WwTerrainaccessor * ta)
 {
 	WwMesh *mesh;
@@ -254,32 +204,34 @@ ww_mesh_new_with_terrain (gdouble lon_max, gdouble lon_min, gdouble lat_max, gdo
 	priv->nv = (nx + 1) * (ny + 1);
 	priv->ni = ((nx + 1) * 2 + 2) * ny;
 
-	multi = mmap_get_vertical_exaggeration ();
+	if (ta == NULL) {
+		multi = 1.0;
+	} else {
+		multi = mmap_get_vertical_exaggeration ();
+	}
 
-	priv->v_deg = mesh_create_deg (lon_min, lat_min, lon_max, lat_max, nx, ny, ta);
+	priv->v_deg = mesh_create_deg (x0, y0, x1, y1, nx, ny, ta);
 	priv->v_xyz = mesh_create_xyz (priv->v_deg, nx, ny, above_surface, multi);
 	priv->index = mesh_create_index (nx, ny);
 
-	priv->t_st = mesh_create_st (nx, ny, WW_MESH_TYPE_NORMAL);
-	priv->type = WW_MESH_TYPE_NORMAL;
-
 	return mesh;
 };
 
+/* tx0, ty0, tx1, ty1 ::: コピー元テクスチャーの範囲
+ * mx0, my0, mx1, my1 ::: コピー元メッシュの範囲
+ * cx0, cy0, cx1, cy1 ::: コピー先の範囲 */
 WwMesh *
-ww_mesh_new_copy (gdouble lon_max, gdouble lon_min, gdouble lat_max, gdouble lat_min,
-				  gint nx, gint ny, gdouble above_surface, gboolean terrain_mapped, WwMesh * mesh_parent,
-				  WwMeshType type)
+ww_mesh_new_copy (gdouble px0, gdouble py0, gdouble px1, gdouble py1,
+				  gdouble cx0, gdouble cy0, gdouble cx1, gdouble cy1,
+				  gint nx, gint ny, gdouble above_surface, gboolean terrain_mapped, WwMesh * mesh_parent)
 {
 	WwMesh *mesh;
 	WwMeshPrivate *priv;
-	WwMeshPrivate *parent_priv;
+	WwMeshPrivate *priv_p;
 	gdouble multi;
-
-	/*
-	   g_print ("ww_mesh_new_copy:%d\n", type);
+	/*  
+	   g_print ("ww_mesh_new_copy\n");
 	 */
-
 	mesh = ww_mesh_new ();
 	priv = WW_MESH_GET_PRIVATE (mesh);
 
@@ -296,14 +248,11 @@ ww_mesh_new_copy (gdouble lon_max, gdouble lon_min, gdouble lat_max, gdouble lat
 		multi = 1.0;
 	}
 
-	parent_priv = WW_MESH_GET_PRIVATE (mesh_parent);
-	priv->v_deg = mesh_create_deg_copy (lon_min, lat_min, lon_max, lat_max, nx, ny, type, parent_priv->v_deg);
+	priv_p = WW_MESH_GET_PRIVATE (mesh_parent);
+	priv->v_deg = mesh_create_deg_copy (px0, py0, px1, py1, cx0, cy0, cx1, cy1, nx, ny, priv_p->v_deg);
 	priv->v_xyz = mesh_create_xyz (priv->v_deg, nx, ny, above_surface, multi);
 	priv->index = mesh_create_index (nx, ny);
 
-	priv->t_st = mesh_create_st (nx, ny, type);
-	priv->type = type;
-
 	return mesh;
 }
 
@@ -324,37 +273,6 @@ ww_mesh_change_vertical_exaggeration (WwMesh * mesh)
 	mesh_change_xyz (priv->v_deg, priv->v_xyz, priv->nx, priv->ny, priv->distance_above_surface, multi);
 }
 
-void
-ww_mesh_change_type (WwMesh * mesh, WwMeshType type)
-{
-	WwMeshPrivate *priv;
-
-	if (mesh == NULL) {
-		return;
-	}
-
-	priv = WW_MESH_GET_PRIVATE (mesh);
-
-	if (priv->type == type) {
-		return;
-	}
-
-	g_free (priv->t_st);
-
-	priv->t_st = mesh_create_st (priv->nx, priv->ny, type);
-	priv->type = type;
-}
-
-WwMeshType
-ww_mesh_get_mesh_type (WwMesh * mesh)
-{
-	WwMeshPrivate *priv;
-
-	priv = WW_MESH_GET_PRIVATE (mesh);
-
-	return priv->type;
-}
-
 void
 ww_mesh_render (WwMesh * mesh, WwMeshRenderType type, gboolean is_in_progress, gint color)
 {
@@ -367,7 +285,6 @@ ww_mesh_render (WwMesh * mesh, WwMeshRenderType type, gboolean is_in_progress, g
 
 	if (is_in_progress == TRUE) {	/* debug 用 */
 		m = 0.5;
-		color *= 0.5;
 	}
 
 	priv = WW_MESH_GET_PRIVATE (mesh);
@@ -375,14 +292,12 @@ ww_mesh_render (WwMesh * mesh, WwMeshRenderType type, gboolean is_in_progress, g
 	if (type == WW_MESH_RENDER_TYPE_TEXTURE) {
 		/* テクスチャーを表示 */
 		glColor3d (1.0, 1.0, 1.0);
-		glTexCoordPointer (2, GL_DOUBLE, 0, priv->t_st);
 		glVertexPointer (3, GL_DOUBLE, 0, priv->v_xyz);
 		glDrawElements (GL_TRIANGLE_STRIP, priv->ni, GL_UNSIGNED_INT, priv->index);
 		/*
 		   glDrawElements (GL_LINE_STRIP, priv->ni, GL_UNSIGNED_INT, priv->index);
 		 */
 
-		/* デバッグ用で四角の表示 */
 		glDisable (GL_TEXTURE_2D);
 		glDisable (GL_DEPTH_TEST);
 
@@ -409,7 +324,7 @@ ww_mesh_render (WwMesh * mesh, WwMeshRenderType type, gboolean is_in_progress, g
 			glEnd ();
 		}
 
-		/*
+		/*  
 		   glLineWidth (1.0);
 		   glBegin (GL_LINE_STRIP);
 		   glVertex3dv (priv->v_xyz);
@@ -464,4 +379,19 @@ ww_mesh_render (WwMesh * mesh, WwMeshRenderType type, gboolean is_in_progress, g
 	} else {
 		;
 	}
+	/*
+	   glDisable (GL_TEXTURE_2D);
+	   glDisable (GL_DEPTH_TEST);
+	   glLineWidth (2.0);
+	   glBegin (GL_LINE_LOOP);
+
+	   glVertex3dv (priv->v_xyz);
+	   glVertex3dv (priv->v_xyz + 3 * priv->nx);
+	   glVertex3dv (priv->v_xyz + 3 * (priv->nx + priv->ny * (priv->nx + 1)));
+	   glVertex3dv (priv->v_xyz + 3 * (0 + priv->ny * (priv->nx + 1)));
+	   glEnd ();
+	   glEnable (GL_DEPTH_TEST);
+	   glEnable (GL_TEXTURE_2D);
+	 */
+
 }
