@@ -108,9 +108,9 @@ ww_mmap_get_type (void)
 		type = g_type_register_static (WW_TYPE_OBJECT, "WwMmap", &info, 0);
 
 		g_type_add_interface_static (type, WW_TYPE_DATA, &ww_data_info);
-        /*
-		g_print ("ww_mmap_get_type\n");
-        */
+		/*
+		   g_print ("ww_mmap_get_type\n");
+		 */
 	}
 
 	return type;
@@ -394,7 +394,7 @@ update_top_level (WwMmap * tile)
 	gdouble lon;
 	gdouble lat;
 
-	camera_get_view (&west, &south, &east, &north);	/* 見える範囲。要修正。極付近は大間違い。 */
+	camera_get_view_narrow (&west, &south, &east, &north);	/* 見える範囲。要修正。極付近は大間違い。 */
 
 	/*
 	   g_print ("update_top_level:old w:%.2f e:%.2f s:%.2f n:%.2f\n", west, east, south, north);
