@@ -101,8 +101,7 @@ static void ww_placename_interface_init (gpointer g_iface, gpointer iface_data);
 /*****************************************************************************/
 
 static void ww_placename_set_attribute (WwData * data, const gchar ** name, const gchar ** value);
-static void ww_placename_set_element (WwData * data, const gchar * element0, const gchar * element1,
-									  const gchar * value);
+static void ww_placename_set_element (WwData * data, const gchar * element0, const gchar * element1, const gchar * value);
 static void ww_placename_set_parent (WwData * data, WwData * parent);
 static void ww_placename_set_child (WwData * data, WwData * child);
 
@@ -140,9 +139,9 @@ ww_placename_get_type (void)
 		type = g_type_register_static (WW_TYPE_OBJECT, "WwPlacename", &info, 0);
 
 		g_type_add_interface_static (type, WW_TYPE_DATA, &ww_data_info);
-        /*
-		g_print ("ww_placename_get_type\n");
-        */
+		/*
+		   g_print ("ww_placename_get_type\n");
+		 */
 	}
 
 	return type;
@@ -505,18 +504,14 @@ ww_placename_update (WwObject * self)
 		gdouble xx0, yy0, xx1, yy1;
 
 		xx0 = wf->west;
-		yy0 = wf->south;
 		xx1 = wf->east;
+		yy0 = wf->south;
 		yy1 = wf->north;
 
-		camera_get_view (&x0, &y0, &x1, &y1);
+		camera_get_view_narrow (&x0, &y0, &x1, &y1);
 
 		/* ４つの辺それぞれに重なりがあるかどうか見ている */
-		if ((yy0 >= y0 && yy0 <= y1 && xx0 <= x1 && xx1 >= x0) || (yy1 >= y0 && yy1 <= y1 && xx0 <= x1 && xx1 >= x0) ||
-			(xx0 >= x0 && xx0 <= x1 && yy0 <= y1 && yy1 >= y0) || (xx1 >= x0 && xx1 <= x1 && yy0 <= y1 && yy1 >= y0) ||
-			/* すっぽり入ってるパターン */
-			(xx0 <= x0 && xx1 >= x1 && yy0 <= y0 && yy1 >= y1)) {
-
+		if (camera_check_overlap (yy1, yy0, xx0, xx1) == TRUE) {
 			gint j;
 
 			if (wf->wwp_place == NULL) {
@@ -529,15 +524,21 @@ ww_placename_update (WwObject * self)
 			for (j = 0; j < wf->n; ++j) {
 				WwpPlace *wp = &(wf->wwp_place[j]);
 
-				if (wp->lon > x0 && wp->lon < x1 && wp->lat > y0 && wp->lat < y1) {
-					wp->visible = TRUE;
+				if (x0 < x1) {
+					if (wp->lon > x0 && wp->lon < x1 && wp->lat > y0 && wp->lat < y1) {
+						wp->visible = TRUE;
+					} else {
+						wp->visible = FALSE;
+					}
 				} else {
-					wp->visible = FALSE;
+					if ((wp->lon > x0 || wp->lon < x1) && wp->lat > y0 && wp->lat < y1) {
+						wp->visible = TRUE;
+					} else {
+						wp->visible = FALSE;
+					}
 				}
 			}
 
-
-
 		} else {
 			if (wf->wwp_place != NULL) {
 				free_wwp_place (wf->wwp_place, wf->n);
@@ -747,6 +748,15 @@ static WinColor wincolor[] = {
 	{"lightgreen", 0x90, 0xee, 0x90},	/* moon */
 	{"lightgray", 0xd3, 0xd3, 0xd3},	/* moon */
 	{"gold", 0xff, 0xd7, 0x00},	/* moon */
+
+	{"darkslateblue", 0x84, 0x70, 0xff},	/* venus */
+	{"crimson", 0xdc, 0x14, 0x3c},	/* venus */
+	{"hotpink", 0xff, 0x69, 0xb4},	/* venus */
+	{"indianred", 0xcd, 0x5c, 0x5c},	/* venus */
+	{"gainsboro", 0xdc, 0xdc, 0xdc},	/* venus */
+	{"bisque", 0xff, 0xe4, 0xc4},	/* venus */
+	{"beige", 0xf5, 0xf5, 0xdc},	/* venus */
+	{"lightsalmon", 0xff, 0xa0, 0x7a},	/* venus */
 };
 
 static void
@@ -763,6 +773,7 @@ wincolor_to_rgb (const gchar * colorname, guint8 * r, guint8 * g, guint8 * b)
 			*g = wincolor[i].g;
 			*b = wincolor[i].b;
 
+			g_free (tmp);
 			return;
 		}
 	}
