@@ -36,6 +36,7 @@
 #include "ww_quadchild.h"
 #include "util.h"
 #include "camera.h"
+#include "simplemath.h"
 
 typedef struct _WwQuadtilePrivate WwQuadtilePrivate;
 struct _WwQuadtilePrivate {
@@ -94,8 +95,7 @@ static void ww_quadtile_interface_init (gpointer g_iface, gpointer iface_data);
 /*****************************************************************************/
 
 static void ww_quadtile_set_attribute (WwData * data, const gchar ** name, const gchar ** value);
-static void ww_quadtile_set_element (WwData * data, const gchar * element0, const gchar * element1,
-									 const gchar * value);
+static void ww_quadtile_set_element (WwData * data, const gchar * element0, const gchar * element1, const gchar * value);
 static void ww_quadtile_set_parent (WwData * data, WwData * parent);
 static void ww_quadtile_set_child (WwData * data, WwData * child);
 
@@ -133,9 +133,9 @@ ww_quadtile_get_type (void)
 		type = g_type_register_static (WW_TYPE_OBJECT, "WwQuadtile", &info, 0);
 
 		g_type_add_interface_static (type, WW_TYPE_DATA, &ww_data_info);
-        /*
-		g_print ("ww_quadtile_get_type\n");
-        */
+		/*
+		   g_print ("ww_quadtile_get_type\n");
+		 */
 	}
 
 	return type;
@@ -305,17 +305,17 @@ get_level (gdouble level_zero_tile_deg, gint number_levels)
 	gint i;
 	gdouble l;
 	gdouble dist = camera_get_dist_target ();
+	gdouble radius = mmap_get_world_radius ();
 
 	/* 視野角55度の時の注視点付近での視野範囲の距離 */
-	l = 2.0 * dist * tan (2.0 * G_PI * 55.0 / 2.0 / 360.0);
+	l = 2.0 * dist * tan (RAD (55.0 / 2.0));
 
 	for (i = 0; i < number_levels; ++i) {
 		gdouble ll;
 		gdouble deg = level_zero_tile_deg / pow (2.0, i);
-		gdouble radius = mmap_get_world_radius ();
 
 		/* 今のレベルでのタイル１枚の辺の距離 */
-		ll = 2.0 * radius * tan (2.0 * G_PI * deg / 2.0 / 360.0);
+		ll = 2.0 * radius * tan (RAD (deg / 2.0));
 
 		/*
 		   g_print("get_level:dist:%.2f   l:%.2f  ll:%.2f\n", dist, l, ll);
@@ -332,33 +332,31 @@ get_level (gdouble level_zero_tile_deg, gint number_levels)
 			return i;
 		}
 	}
-	return number_levels - 1;
+	return number_levels - 1;	/* もっとも詳細なレベル */
 }
 
 static void
-remove_child (WwQuadtile * tile)
+remove_child (WwQuadtile * tile, gint level)
 {
 	WwQuadtilePrivate *priv = WW_QUADTILE_GET_PRIVATE (tile);
 	GSList *sl;
 	/*
-	   g_print ("remove_child\n");
+	   g_print ("quadtile:remove_child\n");
 	 */
 	for (sl = priv->sl_child; sl != NULL; sl = sl->next) {
 		WwQuadchild *child = sl->data;
 
-		ww_quadchild_remove_child (child);
-
+		ww_quadchild_remove_child (child, level);
 	}
 }
 
 static void
-remove_top_level (WwQuadtile * tile)
+remove_top_level (WwQuadtile * tile, gint level)
 {
 	WwQuadtilePrivate *priv = WW_QUADTILE_GET_PRIVATE (tile);
 	GSList *sl;
 	GSList *prev;
 	GSList *next;
-
 	/*
 	   g_print ("remove_top_level\n");
 	 */
@@ -368,9 +366,7 @@ remove_top_level (WwQuadtile * tile)
 
 		next = sl->next;
 
-		if (ww_quadchild_get_need_flag (child) == FALSE) {
-
-			ww_quadchild_is_detached_from_render_list (child);
+		if (ww_quadchild_get_need_flag (child) == FALSE || level < 0) {
 			g_object_unref (child);
 
 			if (prev == NULL) {
@@ -428,13 +424,13 @@ update_top_level (WwQuadtile * tile)
 	gdouble east;
 	gdouble tile_deg = priv->level_zero_tile_size_degrees;
 
-	camera_get_view (&west, &south, &east, &north);	/* 見える範囲。要修正。極付近は大間違い。 */
+	camera_get_view_wide (&west, &south, &east, &north);	/* 見える範囲。要修正。極付近は大間違い。 */
 
 	util_tile_get_view (west, south, east, north, tile_deg, &x0, &y0, &x1, &y1);
 
 	/*
-	   g_print("update_top_level:w:%.2f e:%.2f s:%.2f n:%.2f\n", west, east, south, north);
-	   g_print("update_top_level:(x0,y0)-(x1,y1):(%d,%d)-(%d,%d)\n", x0, y0, x1, y1);
+	   g_print ("update_top_level:w:%.2f e:%.2f s:%.2f n:%.2f\n", west, east, south, north);
+	   g_print ("update_top_level:(x0,y0)-(x1,y1):(%d,%d)-(%d,%d)\n", x0, y0, x1, y1);
 	 */
 
 	/* 範囲が東経180度をまたぐ時は、下の for で使えないので下駄をかます */
@@ -454,10 +450,8 @@ update_top_level (WwQuadtile * tile)
 
 			if (is_created (priv->sl_child, k, j) == FALSE) {
 				WwQuadchild *new_child;
-				/*
-				   g_print("check_lon:###### ok ##### x:%d y:%d\n",k,j);
-				 */
-				new_child = ww_quadchild_new_root (tile, k, j, 0, tile_deg);
+
+				new_child = ww_quadchild_new_simple (tile, NULL, k, j, 0, tile_deg);
 				priv->sl_child = g_slist_prepend (priv->sl_child, new_child);
 			}
 		}
@@ -511,32 +505,29 @@ ww_quadtile_update (WwObject * self)
 	priv = WW_QUADTILE_GET_PRIVATE (tile);
 
 	level = get_level (priv->level_zero_tile_size_degrees, priv->number_levels);
-
 	/*
-	   g_print ("ww_quadtile_update:level(%d):%s\n", level, self->name);
+	   g_print ("ww_quadtile_update:level(target level:%d):%s\n", level, self->name);
 	 */
 
 	ww_quadtile_clear_flag (tile);
 
-	/* 表示すべきでない（表示範囲の４分の1以下）なら update しない。update しないと need_flag が
-	 * 落ちたままなので削除される。 */
-	/* update では、必要なら新しい tile を作ったり、必要フラグを落としたりするが、削除まではしない */
-
 	/*
 	   g_print("wwquadtile_updat:%.2f %.2f %.2f %.2f\n", priv->west, priv->south, priv->east, priv->north);
 	 */
 	if (level >= 0 && camera_check_overlap (priv->north, priv->south, priv->west, priv->east) == TRUE) {
+		/* レベル0 だけを作りなおす。無いものを作り、不要なものの need フラグを落とす。削除はしない。 */
 		update_top_level (tile);
 	}
 	g_mutex_lock (priv->mutex_sl_child);
-	remove_top_level (tile);	/* need_flag の落ちたトップレベルの child を削除する。 */
+	remove_top_level (tile, level);	/* need_flag の落ちたトップレベルの child を削除する。 */
 	g_mutex_unlock (priv->mutex_sl_child);
 
 	/* level1 以上の need_flag はここで付け直す */
 	update_child (tile, level);
 
 	g_mutex_lock (priv->mutex_sl_child);
-	remove_child (tile);		/* need_flag の落ちた child を削除する。 */
+	/* need_flag の落ちた child を削除するとともに、不要になったテクスチャーとメッシュを削除する。 */
+	remove_child (tile, level);
 	g_mutex_unlock (priv->mutex_sl_child);
 
 	new_vertical_exaggeration = mmap_get_vertical_exaggeration ();
