@@ -39,6 +39,7 @@
 #include "ww_mmap.h"
 #include "ww_smap.h"
 #include "ww_texture.h"
+#include "ww_textureloader.h"
 #include "mesh.h"
 #include "camera.h"
 #include "util.h"
@@ -81,10 +82,10 @@ struct _WwSmapPrivate {
 	gchar *path_dds;
 	gchar *path_png;
 	gchar *uri;
-	gboolean initialized;
 	gchar *data;
 	gint size;
 	WwTexture *texture;
+	WwTextureloader *textureloader;
 
 	WwMmap *parent;
 
@@ -126,9 +127,9 @@ ww_smap_get_type (void)
 
 		type = g_type_register_static (G_TYPE_OBJECT, "WwSmap", &info, 0);
 
-        /*
-		g_print ("ww_smap_get_type\n");
-        */
+		/*
+		   g_print ("ww_smap_get_type\n");
+		 */
 	}
 
 	return type;
@@ -184,10 +185,10 @@ ww_smap_init (WwSmap * self)
 	priv->path_dds = NULL;
 	priv->path_png = NULL;
 	priv->uri = NULL;
-	priv->initialized = FALSE;
 	priv->data = NULL;
 	priv->size = 0;
 	priv->texture = NULL;
+	priv->textureloader = NULL;
 
 	priv->parent = NULL;
 
@@ -213,6 +214,11 @@ ww_smap_dispose (GObject * obj)
 
 	if (priv->texture != NULL) {
 		g_object_unref (priv->texture);
+		priv->texture = NULL;
+	}
+	if (priv->textureloader != NULL) {
+		g_object_unref (priv->textureloader);
+		priv->textureloader = NULL;
 	}
 
 	G_OBJECT_CLASS (parent_class)->dispose (obj);
@@ -268,50 +274,16 @@ ww_smap_new (void)
 
 /********************************************************************/
 
-static void
-smap_change_texture_cb (WwTexture * texture, gpointer data)
-{
-	WwSmapPrivate *priv;
-	WwSmap *smap;
-
-	g_return_if_fail (WW_IS_TEXTURE (texture));
-	g_return_if_fail (WW_IS_SMAP (data));
-
-	smap = WW_SMAP (data);
-
-	priv = WW_SMAP_GET_PRIVATE (smap);
-
-	priv->texture = texture;
-	priv->initialized = TRUE;
-
-	g_object_unref (smap);
-
-	glarea_force_update_and_render ();
-}
-
-
-static void
-ww_smap_texture_process (GObject * obj)
+static GdkPixbuf *
+smap_image_manipulate_func (GObject * obj, GdkPixbuf * pixbuf)
 {
 	WwSmap *smap = WW_SMAP (obj);
 	WwSmapPrivate *priv = WW_SMAP_GET_PRIVATE (smap);
-    WwTexture *texture;
-
-	GdkPixbuf *pixbuf;
 	GdkPixbuf *pixbuf_scale;
 	gint texture_size = glarea_get_max_texture_size ();
 	gboolean left;
 
-	g_print ("ww_smap_texture_process:%s\n", priv->name);
-
-	if (priv->is_detached == TRUE) {
-		g_print ("ww_smap_texture_process:detacched:%s\n", priv->name);
-		return;
-	}
-
-	if (priv->data == NULL) {
-		return;
-	}
+	g_print ("smap_image_manipulate_func:%s\n", priv->name);
 
 	if (priv->name[7] == '0') {
 		left = TRUE;
@@ -319,43 +291,11 @@ ww_smap_texture_process (GObject * obj)
 		left = FALSE;
 	}
 
-	disk_save_bin (priv->path_png, priv->data, priv->size);
-	pixbuf = util_bin_to_pixbuf (priv->data, priv->size);
 	pixbuf_scale = util_pixbuf_scale_down (pixbuf, left, texture_size);
 
-	texture = ww_texture_new_pixbuf (pixbuf_scale, priv->path_dds);
-
-	g_object_ref (obj);
-	g_signal_connect (G_OBJECT (texture), "initialized", G_CALLBACK (smap_change_texture_cb), obj);
-	mmap_idle_add_single (ww_texture_gen, G_OBJECT (texture));
-
-	g_object_unref (pixbuf);
-	g_object_unref (pixbuf_scale);
-	g_free (priv->data);
-	priv->data = NULL;
-
+	return pixbuf_scale;
 }
 
-static void
-ww_smap_texture_download (GObject * obj)
-{
-	WwSmap *smap = WW_SMAP (obj);
-	WwSmapPrivate *priv = WW_SMAP_GET_PRIVATE (smap);
-	gint size;
-
-	if (priv->is_detached == TRUE) {
-		g_print ("ww_smap_texture_download:detached:%s\n", priv->name);
-		return;
-	}
-
-	g_print ("ww_smap_texture_download:%s:\n", priv->uri);
-	priv->data = mmapthread_download_get (priv->uri, &size);
-	priv->size = size;
-
-	mmapthread_add_power (ww_smap_texture_process, obj);
-}
-
-
 /* 経度方向は、-1 - 225 と 225 - 451 でとり
  * 緯度方向は、+1 - 151 と 151 - 301 でとる */
 WwSmap *
@@ -369,6 +309,7 @@ ww_smap_new_with_name (WwMmap * parent, gint lon_sec, gint lat_sec, const gchar
 	gchar *path_n;
 	gint dummy;
 	gdouble multi;
+	gint max_texture_size;
 
 	smap = ww_smap_new ();
 	priv = WW_SMAP_GET_PRIVATE (smap);
@@ -394,7 +335,6 @@ ww_smap_new_with_name (WwMmap * parent, gint lon_sec, gint lat_sec, const gchar
 
 	path_v = g_strconcat (mmap_dir_gsi_vn, priv->name, ".v", NULL);
 	path_n = g_strconcat (mmap_dir_gsi_vn, priv->name, ".n", NULL);
-
 	/*
 	   multi = mmap_get_vertical_exaggeration ();
 	 */
@@ -429,48 +369,24 @@ ww_smap_new_with_name (WwMmap * parent, gint lon_sec, gint lat_sec, const gchar
 	   g_print ("ww_smap_new_with_name:uri:%s\n", priv->uri);
 	 */
 
+	max_texture_size = glarea_get_max_texture_size ();
+
 	priv->texture = NULL;
-	priv->initialized = FALSE;
-
-	if (filetest (priv->path_dds) == TRUE) {
-		WwTexture *texture;
-		gint max_texture_size = glarea_get_max_texture_size ();
-
-		texture = ww_texture_new_dds (priv->path_dds, max_texture_size, max_texture_size);
-
-		g_object_ref (smap);
-		g_signal_connect (G_OBJECT (texture), "initialized", G_CALLBACK (smap_change_texture_cb), smap);
-		mmap_idle_add_single (ww_texture_gen, G_OBJECT (texture));
-
-	} else if (filetest (priv->path_png) == TRUE) {
-		WwTexture *texture;
-		GdkPixbuf *pixbuf;
-		GdkPixbuf *pixbuf_scale;
-		gint texture_size = glarea_get_max_texture_size ();
-		gboolean left;
-
-		if (priv->name[7] == '0') {
-			left = TRUE;
-		} else {
-			left = FALSE;
-		}
-
-		pixbuf = disk_load_jpg (priv->path_png);
-		pixbuf_scale = util_pixbuf_scale_down (pixbuf, left, texture_size);
-
-		texture = ww_texture_new_pixbuf (pixbuf_scale, priv->path_dds);
-
-		g_object_ref (smap);
-		g_signal_connect (G_OBJECT (texture), "initialized", G_CALLBACK (smap_change_texture_cb), smap);
-		mmap_idle_add_single (ww_texture_gen, G_OBJECT (texture));
-
-		g_object_unref (pixbuf);
-		g_object_unref (pixbuf_scale);
+	priv->textureloader = ww_textureloader_lookup (priv->uri);
+	if (priv->textureloader == NULL) {
+		WwTexture *texture = ww_texture_new ();
 
+		priv->textureloader = ww_textureloader_new ();
+		ww_textureloader_set_source (priv->textureloader, priv->path_dds, priv->path_png, priv->uri, max_texture_size,
+									 max_texture_size);
+		ww_textureloader_set_texture (priv->textureloader, texture);
+		ww_texture_loader_set_image_manipulate_func (priv->textureloader, smap_image_manipulate_func, smap);
+		ww_textureloader_start (priv->textureloader);
 	} else {
-		mmapthread_add_download2 (priv->uri, ww_smap_texture_download, G_OBJECT (smap));
+		;
 	}
 
+
 	return smap;
 }
 
@@ -491,11 +407,19 @@ ww_smap_clear_flag (WwSmap * smap)
 void
 ww_smap_update (WwSmap * smap)
 {
-	/*
-	   WwSmapPrivate *priv = WW_SMAP_GET_PRIVATE (smap);
-	 */
+	WwSmapPrivate *priv = WW_SMAP_GET_PRIVATE (smap);
 
-	;
+	if (priv->texture == NULL && priv->textureloader != NULL) {
+		priv->texture = ww_textureloader_get_texture (priv->textureloader);
+		if (priv->texture != NULL) {
+			g_object_ref (priv->texture);
+
+			g_object_unref (priv->textureloader);
+			priv->textureloader = NULL;
+
+			glarea_force_update_and_render ();
+		}
+	}
 }
 
 
@@ -524,7 +448,7 @@ ww_smap_render (WwSmap * smap)
 #endif
 	}
 
-	if (ww_texture_is_initialized (priv->texture) == TRUE) {
+	if (priv->texture != NULL) {
 		/* テクスチャーを表示 */
 
 		light_on ();
