@@ -53,10 +53,8 @@ void ww_terrainaccessor_set_higher_resolution_subsets (WwTerrainaccessor * paren
 
 WwTerrainaccessor *ww_terrainaccessor_select (WwTerrainaccessor * ta, gdouble x0, gdouble y0, gdouble x1, gdouble y1);
 
-void ww_terrainaccessor_data_prepare (WwTerrainaccessor * ta, gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx,
-									  gint ny);
-void ww_terrainaccessor_read_data (WwTerrainaccessor * ta, gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx,
-								   gint ny);
+void ww_terrainaccessor_data_prepare (WwTerrainaccessor * ta, gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, gint ny);
+void ww_terrainaccessor_read_data (WwTerrainaccessor * ta, gdouble x0, gdouble y0, gdouble x1, gdouble y1, gint nx, gint ny);
 gdouble ww_terrainaccessor_get_alt (WwTerrainaccessor * ta, gdouble lon, gdouble lat);
 
 void ww_terrainaccessor_debug_print (WwTerrainaccessor * ta);
