@@ -35,6 +35,7 @@
 #include "xmlutil.h"
 #include "ww_object.h"
 #include "layermanager.h"
+#include "simplemath.h"
 
 
 typedef struct _WwWorldPrivate WwWorldPrivate;
@@ -144,15 +145,19 @@ ww_world_dispose (GObject * obj)
 	WwWorldPrivate *priv = WW_WORLD_GET_PRIVATE (self);
 	GSList *sl;
 	/*
-	   g_print ("dispose\n");
+	   g_print ("ww_world_dispose\n");
 	 */
+
 	if (priv->dispose_has_run) {
 		return;
 	}
 	priv->dispose_has_run = TRUE;
 
-	g_object_unref (priv->terrain_accessor);
+	if (priv->terrain_accessor != NULL) {
+		g_object_unref (priv->terrain_accessor);
+	}
 	priv->terrain_accessor = NULL;
+
 	for (sl = priv->sl_layers; sl != NULL; sl = sl->next) {
 		WwObject *layer = sl->data;
 
@@ -163,6 +168,7 @@ ww_world_dispose (GObject * obj)
 	priv->sl_layers = NULL;
 
 	G_OBJECT_CLASS (parent_class)->dispose (obj);
+
 }
 
 static void
@@ -170,10 +176,10 @@ ww_world_finalize (GObject * obj)
 {
 	WwWorld *self = WW_WORLD (obj);
 	WwWorldPrivate *priv = WW_WORLD_GET_PRIVATE (self);
-
 	/*
-	   g_print ("finalize\n");
+	   g_print ("ww_world_finalize\n");
 	 */
+
 	g_free (priv->name);
 	g_free (priv->layer_directory);
 
@@ -278,6 +284,8 @@ ww_world_set_child (WwData * data, WwData * child)
 	WwTerrainaccessor *obj_child = WW_TERRAINACCESSOR (child);
 	WwWorldPrivate *priv = WW_WORLD_GET_PRIVATE (obj);
 
+	g_print ("ww_world_set_child\n");
+
 	priv->terrain_accessor = obj_child;
 }
 
@@ -371,10 +379,10 @@ ww_world_deg_to_xyz (WwWorld * world, gdouble lon, gdouble lat, gdouble alt, gdo
 
 	radius = priv->equatorial_radius;
 
-	r = (radius + alt) * cos (2.0 * G_PI * lat / 360.0);
-	*x = r * cos (2.0 * G_PI * lon / 360.0);
-	*y = r * sin (2.0 * G_PI * lon / 360.0);
-	*z = (radius + alt) * sin (2.0 * G_PI * lat / 360.0);
+	r = (radius + alt) * cos (RAD (lat));
+	*x = r * cos (RAD (lon));
+	*y = r * sin (RAD (lon));
+	*z = (radius + alt) * sin (RAD (lat));
 
 	DB (g_print ("ww_world_deg_to_xyz:x:%.1f y:%.1f z:%.1f\n", *x, *y, *z));
 }
@@ -416,6 +424,9 @@ ww_world_create_layer (WwWorld * world)
 	WwWorldPrivate *priv = WW_WORLD_GET_PRIVATE (world);
 	gchar *files_earth[] = { "Gsi.xml", "Base.xml", "Tiled.xml", "Images.xml", "Placenames.xml" };
 	gchar *files_moon[] = { "Images.xml", "Placenames.xml" };
+	gchar *files_venus[] = { "Images.xml", "Placenames.xml" };
+	gchar *files_mars[] = { "Images.xml" };
+	gchar *files_jupiter[] = { "Images.xml" };
 	gchar **files;
 	gint i, n;
 
@@ -424,9 +435,18 @@ ww_world_create_layer (WwWorld * world)
 	if (g_strcasecmp (priv->name, "Earth") == 0) {
 		files = files_earth;
 		n = sizeof files_earth / sizeof (gchar *);
-	} else {
+	} else if (g_strcasecmp (priv->name, "Moon") == 0) {
 		files = files_moon;
 		n = sizeof files_moon / sizeof (gchar *);
+	} else if (g_strcasecmp (priv->name, "Venus") == 0) {
+		files = files_venus;
+		n = sizeof files_venus / sizeof (gchar *);
+	} else if (g_strcasecmp (priv->name, "Mars") == 0) {
+		files = files_venus;
+		n = sizeof files_mars / sizeof (gchar *);
+	} else if (g_strcasecmp (priv->name, "Jupiter") == 0) {
+		files = files_jupiter;
+		n = sizeof files_jupiter / sizeof (gchar *);
 	}
 
 	for (i = 0; i < n; ++i) {
