@@ -199,7 +199,6 @@ xmlutil_get_state (XmlData * xmldata)
 	/*
 	   g_print("xmldata:%p:   sl_state:%p:\n", xmldata, xmldata->sl_state);
 	 */
-
 	gint *state = g_slist_nth_data (xmldata->sl_state, 0);
 
 	return *state;
@@ -302,8 +301,8 @@ xmlutil_prepare (const gchar * filename, GMarkupParser * parser)
 
 	context = g_markup_parse_context_new (parser, 0, xmldata, NULL);
 
-	if (!g_markup_parse_context_parse (context, text, length, NULL)) {
-		g_print ("error:g_markup_parse_context_parse\n");
+	if (!g_markup_parse_context_parse (context, text, length, &err)) {
+		g_print ("error:g_markup_parse_context_parse:%s\n", err->message);
 
 		g_markup_parse_context_free (context);
 		exit (1);
@@ -326,8 +325,7 @@ xmlutil_prepare (const gchar * filename, GMarkupParser * parser)
 
 static void
 start_element_handler (GMarkupParseContext * context, const gchar * element_name,
-					   const gchar ** attribute_names, const gchar ** attribute_values,
-					   gpointer user_data, GError ** err)
+					   const gchar ** attribute_names, const gchar ** attribute_values, gpointer user_data, GError ** err)
 {
 	XmlData *xmldata = user_data;
 	GType gtype;
